"""Python Provider for TestMethod

Instruments the CIM class TestMethod

"""

import pywbem
import random
from pywbem.cim_provider2 import CIMProvider2

g_insts = {}

class TestMethodProvider(CIMProvider2):
    """Instrument the CIM class TestMethod 

    Class with several methods to test method provider capabilities.
    
    """

    def __init__ (self, env):
        logger = env.get_logger()
        logger.log_debug('Initializing provider %s from %s.... num g_insts=%d' \
                % (self.__class__.__name__, __file__, len(g_insts)))

    def get_instance(self, env, model):
        logger = env.get_logger()
        logger.log_debug('Entering %s.get_instance()' \
                % self.__class__.__name__)

        try:
            inst = g_insts[model['id']]
            #inst = g_insts[model.path['id']]
        except KeyError:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        
        #model['id'] = model.path['id']
        model['p_sint32']=inst[1]
        model['p_str']=pywbem.CIMProperty('p_str', type='string', value=inst[0])
        #model['p_str']=inst[0]
        return model

    def enum_instances(self, env, model, keys_only):
        logger = env.get_logger()
        logger.log_debug('Entering %s.enum_instances()' \
                % self.__class__.__name__)

        '''
        ch = env.get_cimom_handle()
        try:
            e = ch.EnumerateInstanceNames("root/cimv2", "TestAssoc_User")
            for i in e:
                print i
        except pywbem.CIMError, e:
            print e
        '''

        '''
        ch = env.get_cimom_handle()

        try:
            ch.bummer()
        except pywbem.CIMError, e:
            print e
        '''

        l = env.get_logger()
        l.log_debug("hello");

        #except CIMError, e:

        for key in g_insts.keys():
            model['id'] = key
            model.path['id'] = key
            if keys_only:
                yield model
            else:
                try:
                    yield self.get_instance(env, model)
                except pywbem.CIMError, (num, msg):
                    if num not in (pywbem.CIM_ERR_NOT_FOUND, 
                                   pywbem.CIM_ERR_ACCESS_DENIED):
                        raise

    def set_instance(self, env, instance, modify_existing):
        logger = env.get_logger()
        logger.log_debug('Entering %s.set_instance()' \
                % self.__class__.__name__)

        if modify_existing:
            if instance['id'] not in g_insts:
                raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        else:
            g_insts[instance['id']] = [None,None]
        try:
            s = instance['p_str']
        except KeyError:
            s = g_insts[instance['id']][0]
        try: 
            i = instance['p_sint32']
        except KeyError:
            i = g_insts[instance['id']][1]
        g_insts[instance['id']] = [s,i]
        return instance

    def delete_instance(self, env, instance_name):
        logger = env.get_logger()
        logger.log_debug('Entering %s.delete_instance()' \
                % self.__class__.__name__)

        try:
            del g_insts[instance_name['id']]
        except KeyError:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        
    def cim_method_mkunichar(self, env, object_name, 
                             param_c):

        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_mkunichar()' \
                % self.__class__.__name__)

        # TODO do something
        raise pywbem.CIMError(pywbem.CIM_ERR_METHOD_NOT_AVAILABLE) # Remove to implemented
        out_params = {}
        rval = None # TODO (type pywbem.Char16)
        return (rval, out_params)
        
    def cim_method_mkunistr_char16(self, env, object_name, 
                                   param_carr):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_mkunistr_char16()' \
                % self.__class__.__name__)

        # TODO do something
        raise pywbem.CIMError(pywbem.CIM_ERR_METHOD_NOT_AVAILABLE) # Remove to implemented
        out_params = {}
        rval = None # TODO (type unicode)
        return (rval, out_params)
        
    def cim_method_strsplit(self, env, object_name, 
                            param_str,
                            param_sep):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_strsplit()' \
                % self.__class__.__name__)

        elems = param_str.split(param_sep)
        out_params = [
                pywbem.CIMParameter('nelems', 'sint32', 
                    value=pywbem.Sint32(len(elems))), 
                pywbem.CIMParameter('elems', 'string', value=elems) ]
        rval = True
        return (rval, out_params)
        
    def cim_method_strcat(self, env, object_name,
                          param_strs,
                          param_sep):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_strcat()' \
                % self.__class__.__name__)

        out_params = {}
        rval = param_sep.join(param_strs)
        return (rval, out_params)
        
    def cim_method_mkunichararray(self, env, object_name, 
                                  param_inarr):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_mkunichararray()' \
                % self.__class__.__name__)

        # TODO do something
        raise pywbem.CIMError(pywbem.CIM_ERR_METHOD_NOT_AVAILABLE) # Remove to implemented
        out_params = {}
        #out_params['outarr'] = # TODO (type [pywbem.Char16,])
        rval = None # TODO (type bool)
        return (rval, out_params)
        
    def cim_method_mkunistr_sint8(self, env, object_name,
                                  param_carr):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_mkunistr_sint8()' \
                % self.__class__.__name__)

        rval = ''
        for i in param_carr:
            rval += chr(i)
        out_params = {}
        return (rval, out_params)

    def minmedmax(self, dt, env, object_name,  param_numlist):
        l = param_numlist
        l.sort()

        lmin = l[0]
        lmax = l[-1]
        ln = len(l)
        if ln % 2 == 0:
            lmed = dt((l[(ln / 2) - 1] + l[(ln / 2)]) / 2)
        else:
            lmed = dt(l[ln / 2])

        out_params = [
                pywbem.CIMParameter('max', lmax.cimtype, value=lmax),
                pywbem.CIMParameter('min', lmin.cimtype, value=lmin),
                pywbem.CIMParameter('med', lmed.cimtype, value=lmed)
                ]

        rval = True
        return (rval, out_params)
        
    def cim_method_minmedmax_real64(self, env, object_name,
                                    param_numlist):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_minmedmax_real64()' \
                % self.__class__.__name__)

        return self.minmedmax(pywbem.Real64, env, object_name,  param_numlist)
        
    def cim_method_minmedmax_real32(self, env, object_name,
                                    param_numlist):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_minmedmax_real32()' \
                % self.__class__.__name__)

        return self.minmedmax(pywbem.Real32, env, object_name,  param_numlist)
        
    def cim_method_minmedmax_uint8(self, env, object_name,
                                   param_numlist):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_minmedmax_uint8()' \
                % self.__class__.__name__)

        return self.minmedmax(pywbem.Uint8, env, object_name,  param_numlist)
        
    def cim_method_minmedmax_sint64(self, env, object_name,
                                    param_numlist):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_minmedmax_sint64()' \
                % self.__class__.__name__)

        return self.minmedmax(pywbem.Sint64, env, object_name,  param_numlist)
        
    def cim_method_minmedmax_sint16(self, env, object_name,
                                    param_numlist):

        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_minmedmax_sint16()' \
                % self.__class__.__name__)

        return self.minmedmax(pywbem.Sint16, env, object_name,  param_numlist)
        
    def cim_method_minmedmax_uint16(self, env, object_name,
                                    param_numlist):

        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_minmedmax_uint16()' \
                % self.__class__.__name__)

        return self.minmedmax(pywbem.Uint16, env, object_name,  param_numlist)
        
    def cim_method_minmedmax_uint64(self, env, object_name,
                                    param_numlist):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_minmedmax_uint64()' \
                % self.__class__.__name__)

        return self.minmedmax(pywbem.Uint64, env, object_name,  param_numlist)
        
    def cim_method_minmedmax_sint8(self, env, object_name, 
                                   param_numlist):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_minmedmax_sint8()' \
                % self.__class__.__name__)

        return self.minmedmax(pywbem.Sint8, env, object_name,  param_numlist)
        
    def cim_method_minmedmax_sint32(self, env, object_name, 
                                    param_numlist):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_minmedmax_sint32()' \
                % self.__class__.__name__)

        return self.minmedmax(pywbem.Sint32, env, object_name,  param_numlist)
        
    def cim_method_minmedmax_uint32(self, env, object_name, 
                                    param_numlist):

        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_minmedmax_uint32()' \
                % self.__class__.__name__)

        return self.minmedmax(pywbem.Uint32, env, object_name,  param_numlist)
        
    def cim_method_getdate(self, env, object_name, 
                           param_datestr):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_getdate()' \
                % self.__class__.__name__)

        out_params = {}
        rval = pywbem.CIMDateTime(param_datestr)
        return (rval, out_params)
        
    def cim_method_getdates(self, env, object_name, 
                            param_datestrs):

        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_getdates()' \
                % self.__class__.__name__)

        out_params = {}
        elems = [pywbem.CIMDateTime(s) for s in param_datestrs]
        out_params = [
                pywbem.CIMParameter('nelems', 'sint32', 
                    value=pywbem.Sint32(len(elems))),
                pywbem.CIMParameter('elems','datetime', value=elems)
                ]
        rval = True
        return (rval, out_params)
        
    def cim_method_getintprop(self, env, object_name):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_getintprop()' \
                % self.__class__.__name__)

        if not hasattr(object_name, 'keybindings'):
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER)
        try:
            inst = g_insts[object_name['id']]
        except KeyError:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        out_params = {}
        rval = inst[1]
        return (rval, out_params)
        
    def cim_method_getstrprop(self, env, object_name):

        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_getstrprop()' \
                % self.__class__.__name__)

        if not hasattr(object_name, 'keybindings'):
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER)
        try:
            inst = g_insts[object_name['id']]
        except KeyError:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        out_params = {}
        rval = inst[0]
        return (rval, out_params)
        
    def cim_method_setstrprop(self, env, object_name,
                              param_value):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_setstrprop()' \
                % self.__class__.__name__)

        if not hasattr(object_name, 'keybindings'):
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER)
        try:
            inst = g_insts[object_name['id']]
        except KeyError:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)

        rval = inst[0]
        inst[0] = param_value
        out_params = {}
        return (rval, out_params)
        
    def cim_method_setintprop(self, env, object_name,
                              param_value):

        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_setintprop()' \
                % self.__class__.__name__)

        if not hasattr(object_name, 'keybindings'):
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER)
        try:
            inst = g_insts[object_name['id']]
        except KeyError:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)

        rval = inst[1]
        inst[1] = param_value
        out_params = {}
        return (rval, out_params)

    def genrandlist_i(self, dt, env, object_name,  param_nelems, 
                                param_lo, param_hi):
        cnt = 0
        l = []
        while cnt < param_nelems:
            cnt+= 1
            try:
                l.append(dt(random.randint(param_lo, param_hi)))
            except ValueError:
                # Range was probably zero
                l.append(dt(param_lo))
        out_params = []
        out_params.append(pywbem.CIMParameter('lo', param_lo.cimtype, 
            value=param_lo))
        out_params.append(pywbem.CIMParameter('hi', param_hi.cimtype, 
            value=param_hi))
        tmpt = dt(0)
        out_params.append(pywbem.CIMParameter('nlist', tmpt.cimtype, value=l))
        rval = True
        return (rval, out_params)
        
    def cim_method_genrandlist_sint64(self, env, object_name,
                                      param_nelems,
                                      param_lo,
                                      param_hi):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_genrandlist_sint64()' \
                % self.__class__.__name__)
        return self.genrandlist_i(pywbem.Sint64, env, object_name,  
                                    param_nelems, param_lo, param_hi)

        
    def cim_method_genrandlist_uint16(self, env, object_name,
                                      param_nelems,
                                      param_lo,
                                      param_hi):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_genrandlist_uint16()' \
                % self.__class__.__name__)
        return self.genrandlist_i(pywbem.Uint16, env, object_name, 
                                    param_nelems, param_lo, param_hi)

        
    def cim_method_genrandlist_uint32(self, env, object_name,
                                      param_nelems,
                                      param_lo,
                                      param_hi):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_genrandlist_uint32()' \
                % self.__class__.__name__)
        return self.genrandlist_i(pywbem.Uint32, env, object_name, 
                                    param_nelems, param_lo, param_hi)

    def cim_method_genrandlist_sint8(self, env, object_name,
                                     param_nelems,
                                     param_lo,
                                     param_hi):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_genrandlist_sint8()' \
                % self.__class__.__name__)
        return self.genrandlist_i(pywbem.Sint8, env, object_name, 
                                    param_nelems, param_lo, param_hi)

    def cim_method_genrandlist_uint8(self, env, object_name,
                                     param_nelems,
                                     param_lo,
                                     param_hi):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_genrandlist_uint8()' \
                % self.__class__.__name__)
        return self.genrandlist_i(pywbem.Uint8, env, object_name, 
                                    param_nelems, param_lo, param_hi)

    def cim_method_genrandlist_uint64(self, env, object_name,
                                      param_nelems,
                                      param_lo,
                                      param_hi):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_genrandlist_uint64()' \
                % self.__class__.__name__)
        return self.genrandlist_i(pywbem.Uint64, env, object_name, 
                                    param_nelems, param_lo, param_hi)

    def cim_method_genrandlist_sint32(self, env, object_name,
                                      param_nelems,
                                      param_lo,
                                      param_hi):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_genrandlist_sint32()' \
                % self.__class__.__name__)
        return self.genrandlist_i(pywbem.Sint32, env, object_name, 
                                    param_nelems, param_lo, param_hi)

    def cim_method_genrandlist_sint16(self, env, object_name,
                                      param_nelems,
                                      param_lo,
                                      param_hi):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_genrandlist_sint16()' \
                % self.__class__.__name__)
        return self.genrandlist_i(pywbem.Sint16, env, object_name, 
                                    param_nelems, param_lo, param_hi)

    def genrandlist_r(self, dt, env, object_name, param_nelems, 
                            param_lo, param_hi):
        cnt = 0
        l = []
        range = param_hi - param_lo
        while cnt < param_nelems:
            cnt+= 1
            l.append(dt(random.random() * range + param_lo))
        out_params = []
        out_params.append(pywbem.CIMParameter('lo', param_lo.cimtype, 
            value=param_lo))
        out_params.append(pywbem.CIMParameter('hi', param_hi.cimtype, 
            value=param_hi))
        tmpt = dt(0)
        out_params.append(pywbem.CIMParameter('nlist', tmpt.cimtype, 
            value=l))
        rval = True
        return (rval, out_params)

    def cim_method_genrandlist_real64(self, env, object_name,
                                      param_nelems,
                                      param_lo,
                                      param_hi):

        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_genrandlist_real64()' \
                % self.__class__.__name__)
        return self.genrandlist_r(pywbem.Real64, env, object_name,  
                                    param_nelems, param_lo, param_hi)

        
    def cim_method_genrandlist_real32(self, env, object_name, 
                                      param_nelems,
                                      param_lo,
                                      param_hi):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_genrandlist_real32()' \
                % self.__class__.__name__)
        return self.genrandlist_r(pywbem.Real32, env, object_name,  
                                    param_nelems, param_lo, param_hi)

    def genrand_i(self, dt, env, object_name, 
                                  param_max,
                                  param_min):
        out_params = {}
        out_params = [pywbem.CIMParameter('success', 'boolean', value=True)]
        rval = param_min
        if param_min < param_max:
        	rval = dt(random.randint(param_min, param_max))
        return (rval, out_params)

    def cim_method_genrand_uint64(self, env, object_name, 
                                  param_max,
                                  param_min):

        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_genrand_uint64()' \
                % self.__class__.__name__)

        return self.genrand_i(pywbem.Uint64, env, object_name,  
                param_max, param_min)
        
    def cim_method_genrand_sint8(self, env, object_name, 
                                 param_max,
                                 param_min):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_genrand_sint8()' \
                % self.__class__.__name__)
        return self.genrand_i(pywbem.Sint8, env, object_name,  
                param_max, param_min)

    def cim_method_genrand_sint32(self, env, object_name, 
                                  param_max,
                                  param_min):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_genrand_sint32()' \
                % self.__class__.__name__)
        return self.genrand_i(pywbem.Sint32, env, object_name,  
                param_max, param_min)

    def cim_method_genrand_uint32(self, env, object_name, 
                                  param_max,
                                  param_min):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_genrand_uint32()' \
                % self.__class__.__name__)
        return self.genrand_i(pywbem.Uint32, env, object_name,  
                param_max, param_min)
        
    def cim_method_genrand_sint16(self, env, object_name, 
                                  param_max,
                                  param_min):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_genrand_sint16()' \
                % self.__class__.__name__)
        return self.genrand_i(pywbem.Sint16, env, object_name,  
                param_max, param_min)

    def cim_method_genrand_uint16(self, env, object_name,
                                  param_max,
                                  param_min):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_genrand_uint16()' \
                % self.__class__.__name__)
        return self.genrand_i(pywbem.Uint16, env, object_name,  
                param_max, param_min)

    def cim_method_genrand_uint8(self, env, object_name, 
                                 param_max,
                                 param_min):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_genrand_uint8()' \
                % self.__class__.__name__)
        return self.genrand_i(pywbem.Uint8, env, object_name,  
                param_max, param_min)
        
    def cim_method_genrand_sint64(self, env, object_name, 
                                  param_max,
                                  param_min):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_genrand_sint64()' \
                % self.__class__.__name__)
        return self.genrand_i(pywbem.Sint64, env, object_name,  
                param_max, param_min)

    def genrand_r(self, dt, env, object_name, 
                                  param_max,
                                  param_min):
        out_params = [pywbem.CIMParameter('success', 'boolean', value=True)]
        range = param_max - param_min
        rval = dt(random.random() * range + param_min)
        return (rval, out_params)

    def cim_method_genrand_real32(self, env, object_name, 
                                  param_max,
                                  param_min):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_genrand_real32()' \
                % self.__class__.__name__)
        return self.genrand_r(pywbem.Real32, env, object_name,  
                param_max, param_min)

    def cim_method_genrand_real64(self, env, object_name, 
                                  param_max,
                                  param_min):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_genrand_real64()' \
                % self.__class__.__name__)
        return self.genrand_r(pywbem.Real64, env, object_name,  
                param_max, param_min)

    def cim_method_delobject(self, env, object_name, 
                             param_path):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_delobject()' \
                % self.__class__.__name__)

        id = param_path['id']
        if id not in g_insts:
            rval = pywbem.Sint32(pywbem.CIM_ERR_NOT_FOUND)
        else:
            del g_insts[id]
            rval = pywbem.Sint32(0)
        return (rval, {})

    def cim_method_delobjects(self, env, object_name, 
                              param_paths):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_delobjects()' \
                % self.__class__.__name__)
        rval = pywbem.Sint32(0)
        for path in param_paths:
            id = path['id']
            if id not in g_insts:
                rval = pywbem.Sint32(pywbem.CIM_ERR_NOT_FOUND)
            else:
                del g_insts[id]
        return (rval, {})
        
    def cim_method_getobjectpath(self, env, object_name):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_getobjectpath()' \
                % self.__class__.__name__)

        if not g_insts:
            path = None
            rval = pywbem.Sint32(pywbem.CIM_ERR_NOT_FOUND)
        else:
            path = pywbem.CIMInstanceName('Test_Method', 
                    namespace=object_name.namespace,
                    keybindings={'id':g_insts.keys()[0]})
            rval = pywbem.Sint32(0)
        out_params = [pywbem.CIMParameter('path', 'reference', value=path)]
        return (rval, out_params)

    def cim_method_numinsts(self, env, object_name):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_getobjectpath()' \
                % self.__class__.__name__)

        return (pywbem.Sint32(len(g_insts)), [])

    def cim_method_getobjectpaths(self, env, object_name):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_getobjectpaths()' \
                % self.__class__.__name__)

        paths = []
        for key in g_insts.keys():
            paths.append(pywbem.CIMInstanceName('Test_Method', 
                    namespace=object_name.namespace,
                    keybindings={'id':key}))
        out_params = {}
        out_params = [pywbem.CIMParameter('paths', 'reference', value=paths)]
        rval = pywbem.Sint32(0)
        return (rval, out_params)

    def cim_method_numinsts(self, env, object_name):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_numinsts()' \
                % self.__class__.__name__)

        return (pywbem.Sint32(len(g_insts)), {})
    
    def cim_method_createobject(self, env, object_name, param_inst):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_createobject()' \
                % self.__class__.__name__)

        g_insts[param_inst['id']] = [param_inst['p_str'], 
                                     param_inst['p_sint32']]

        out_params = []
        rval = pywbem.Sint32(0)
        return (rval, out_params)

    def cim_method_createobjects(self, env, object_name, param_insts):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_createobject()' \
                % self.__class__.__name__)

        for inst in param_insts:
            g_insts[inst['id']] = [inst['p_str'], inst['p_sint32']]

        out_params = []
        rval = pywbem.Sint32(0)
        return (rval, out_params)

    def cim_method_getobjects(self, env, object_name):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_getobjects()' \
                % self.__class__.__name__)

        insts = []
        for kn, val in g_insts.items():
            inst = pywbem.CIMInstance('Test_Method', 
                    properties={'id':kn, 'p_str':val[0], 'p_sint32':val[1]})
            inst.path = pywbem.CIMInstanceName('Test_Method', 
                namespace="root/cimv2", keybindings={'id':kn})
            insts.append(inst)

        out_params = [pywbem.CIMParameter('insts', 'instance', value=insts)]
        rval = pywbem.Sint32(0)
        return (rval, out_params)

    def cim_method_getbrokername(self, env, object_name):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_getbrokername()' \
                % self.__class__.__name__)

        out_params = []
        rval = env.get_cimom_handle().broker.name()
        return (rval, out_params)
        
## end of class TestMethodProvider

def get_providers(env): 
    testmethod_prov = TestMethodProvider(env)  
    return {'Test_Method': testmethod_prov} 



#!/usr/bin/env python

import pywbem
from lib import wbem_connection
import unittest
import optparse
import os
import time
from socket import getfqdn
from subprocess import Popen
import signal
import sys

import threading



conn = None

#This test requires the usage of elementtree

_g_opts = None
_g_args = None


################################################################################
class UpcallAtomTest(unittest.TestCase):

    global conn


    def setUp(self):
        unittest.TestCase.setUp(self)
        self.conn = conn
        self.conn.debug = True
        self._verbose = _g_opts.verbose
        self._dbgPrint()


    def tearDown(self):
        unittest.TestCase.tearDown(self)


    def _dbgPrint(self, msg=''):
        if self._verbose:
            if len(msg):
                print('\t -- %s --' % msg)
            else:
                print('')

    def test_a_upcalls_all(self):
        rv,outs = self.conn.InvokeMethod('getBrokerName', 'Test_UpcallAtom')
        if rv == 'Pegasus':
            print '\n*** Broker detected as Pegasus.  Skipping upcall test.'
            print 'Re-enable when Pegasus is fixed.' 
            return
        rv,outs = self.conn.InvokeMethod('test_all_upcalls', 'Test_UpcallAtom')
        self.assertEquals(rv, 'Success!')
        self.assertFalse(outs)


def get_unit_test():
    return UpcallAtomTest


if __name__ == '__main__':
    parser = optparse.OptionParser()
    wbem_connection.getWBEMConnParserOptions(parser)
    parser.add_option('--verbose', '', action='store_true', default=False,
            help='Show verbose output')
    parser.add_option('--op', '', action='store_true', default=False,
            help='Use OpenPegasus UDS Connection')
    parser.add_option('--level',
            '-l',
            action='store',
            type='int',
            dest='dbglevel',
            help='Indicate the level of debugging statements to display (default=2)',
            default=2)
    _g_opts, _g_args = parser.parse_args()
    
    if _g_opts.op:
        conn = pywbem.PegasusUDSConnection()
    else:
        conn = wbem_connection.WBEMConnFromOptions(parser)
    
    suite = unittest.makeSuite(UpcallAtomTest)
    unittest.TextTestRunner(verbosity=_g_opts.dbglevel).run(suite)




import sys
import pywbem
from socket import getfqdn

_verbose = False

def _print (msg, indent=0):
    if _verbose is True:
        for x in xrange(indent):
            sys.__stdout__.write('\t')
        print "\t%s" %msg

###########################################################################
def association_sanity_check(pyunit, 
                             conn, 
                             assocClassName, 
                             verbose = False):
    instance_sanity_check(pyunit, conn, assocClassName, verbose)
    # this does most of the association tests:
    # enumerateInstances
    # enumerateInstanceNames
    # count of each is =
    # elements of each is =
    # getInstance from association instance name
    # getInstance from 'antecedent' and 'dependent' of association
    # Associators/AssociatorNames/References/ReferenceNames

    #print #get some blank lines before we start outputing from here
    global _verbose
    _verbose = verbose
    refClasses={}

    _print("")
    _print("")

    klass=conn.GetClass(assocClassName, LocalOnly=False)
    for propName,prop in klass.properties.items():
        if prop.reference_class is not None:
            refClasses[propName]=prop.reference_class

    tmpStr = ""
    for k,v in refClasses.items():
        tmpStr += '\n\t\t\t%s: %s' %(k,v)
    _print("Entering AssociationSanityCheck.\n\t\tAssocClassName: %s\n\t\t"
           "RefClasses: %s " %(assocClassName, tmpStr),0)

    #get assoc name for reference object check
    instNames = conn.EnumerateInstanceNames(assocClassName)

    _print("Reference Objects Legit?",0)
    #make sure reference objects are legit, etc
    for instName in instNames:
        _print("legit for %s" %instName,2)
        # try get instance
        try:
            _print("Checking 'GetInstance' on the association",1)
            testInst = conn.GetInstance(instName)
            if testInst.path != instName:
                pyunit.fail("In GetInstance, got an instance, but not the "
                            "one I was expecting")
        except pywbem.CIMError, arg:
            pyunit.fail("Got unexected error trying to 'GetInstance' on the "
                        "association: %s" %arg)
        # try get 'reference objects'
        try:
            for rolename in refClasses.keys():
                _print("Checking 'GetInstance' on the association's %s" %rolename,1)
                testInst = conn.GetInstance(instName[rolename])
                if testInst.path != instName[rolename]:
                    pyunit.fail("In GetInstance, got an instance, but not the "
                                "one I was expecting")
        except pywbem.CIMError, arg:
            pyunit.fail("Got unexected error trying to 'GetInstance' on the "
                        "association's %s: %s" %(rolename,arg))

    # Associators/AssociatorNames/References/ReferenceNames

    roles = refClasses.keys()
    # 
    # go through refClasses ('role'='ClassName')

    # Associators
    for role,roleClassName in refClasses.items():
        revAssocRoleName = roles[(roles.index(role)+1)%2]
        revAssocClassName = refClasses[revAssocRoleName]

        _print("")
        _print("Test Associators from RoleClassName: %s (Role: %s)" %(roleClassName, role),0)
        #start by getting first instance name of classname
        testNames = conn.EnumerateInstanceNames(roleClassName)
        #NOTE: Could change this to :
        #for testName in testNames
        # but that could create loads of iterations, depending on the provider
        testName = testNames[0]
#       if testName.host is None:
#            testName.host = getfqdn()
        testName.host = None
        _print("Using first instance of class: %s:" 
               %(roleClassName),1)
        _print(testName, 2)

        #First use Associators with AssocClass
        #use testName to call associators
        _print("Calling Associators with AssocClass=%s" %assocClassName,1)
        assocs = conn.Associators(testName, AssocClass=assocClassName)
        #now use testNameto call associatorNames
        _print("Calling Associator Names with AssocClass=%s" %assocClassName,1)
        assocNames = conn.AssociatorNames(testName, AssocClass=assocClassName)
        
        _print("Same Count?",1)
        #number returned by assocs and assocNames should match
        pyunit.failUnlessEqual(len(assocs), len(assocNames), 
                             "Number of objects returned by Associators(%d) "
                             "and AssociatorNames(%d) doesn't match" 
                             %(len(assocs),len(assocNames)))
        #so should their contents:
        _print("Same objects?",1)
        #first normalize them all by removing all hostnames
        _print("Normalizing")
        for assoc in assocs:
            assoc.path.host = None
        for assocName in assocNames:
            assocName.host = None
        for assoc in assocs:
            if assoc.path not in assocNames:
                pyunit.fail("Associators returned an object %s not returned "
                            "by AssociatorNames" %assoc.path)

        _print("Objects legit?",1)
        for assocName in assocNames:
            try:
                chkInst = conn.GetInstance(assocName)
                if chkInst.path != assocName:
                    pyunit.fail("In GetInstance, got an instance, but not the "
                                "one I was expecting")
            except pywbem.CIMError, arg:
                pyunit.fail("Got unexected error trying to 'GetInstance' on the "
                            "associated object: %s" %arg)

        #for each associator name, we should be able to reverse the association and get at least the starting object
        _print("")
        _print("Check Reverse Associators with AssocClass",1)
        for assocName in assocNames:
            #do associatornames
            _print("Calling Associator Names",2)
            revAssocNames = conn.AssociatorNames(assocName, AssocClass=assocClassName)
            for name in revAssocNames:
                name.host = None
                if not pywbem.is_subclass(conn,
                        'root/cimv2',
                        sub = name.classname,
                        super = roleClassName):
                    pyunit.fail("reverse associator names returned an object type other than %s: %s" %(roleClassName, name.classname))
            _print("Returned original object?",3)
            if testName not in revAssocNames:
                pyunit.fail("Reverse Associator namess from [%s] didn't return the original object: [%s]... returned: [%s]" %(assocName, testName, name))

            #now do associators
            _print("Calling Associators",2)
            revAssocs = conn.Associators(assocName, AssocClass=assocClassName)

            _print("Same Count?",2)
            pyunit.failUnlessEqual(len(revAssocNames), len(revAssocs), 
                             "Number of reverse associator names (%d) and reverse associators (%d) "
                             "returned don't match." %(len(revAssocNames), len(revAssocs)))

            _print("Same Objects?",2)
            # make sure names match... 
            for assoc in revAssocs:
                assoc.path.host = None
                if not pywbem.is_subclass(conn,
                        'root/cimv2',
                        sub = assoc.classname,
                        super = roleClassName):
                    pyunit.fail("reverse associators returned an object type other than %s: %s" %(roleClassname, assoc.classname))
                if assoc.path not in revAssocNames:
                    pyunit.fail("Reverse Associators from [%s] returned an object not returned by reverse associator names %s" %(assocName, assoc.path))

        _print("Check Reverse Associators with ResultClass",1)
        #use testNameto call associatorNames
        _print("Calling Associator Names ",2)
        revAssocNames = conn.AssociatorNames(testName, ResultClass=revAssocClassName)
        #Now use Associators with ResultClass
        _print("Calling Associators",2)
        revAssocs = conn.Associators(testName, ResultClass=revAssocClassName)
        
        _print("Same Count?",2)
        #number returned by assocs and assocNames should match
        pyunit.failUnlessEqual(len(revAssocs), len(revAssocNames), 
                             "Number of objects returned by Associators(%d) "
                             "and AssociatorNames(%d) doesn't match" 
                             %(len(revAssocs),len(revAssocNames)))
        #so should their contents:
        _print("Same Objects?",2)
        for rAssocName in revAssocNames:
            rAssocName.host = None
        for revAssoc in revAssocs:
            revAssoc.path.host = None
            if revAssoc.path not in revAssocNames:
                pyunit.fail("Associators returned an object %s not returned "
                            "by AssociatorNames" %revAssoc.path)



        _print("Test References with ResultClass = %s" %assocClassName,0)
        _print("Calling ReferenceNames ",1)
        refNames = conn.ReferenceNames(testName, ResultClass=assocClassName)
        _print("Calling References ",1)
        refs = conn.References(testName, ResultClass=assocClassName)
        _print("Same Count?",1)
        pyunit.failUnlessEqual(len(refNames), len(refs), 
                             "Number of objects returned by References(%d) "
                             "and ReferenceNames(%d) doesn't match" 
                             %(len(refs), len(refNames)))
        _print("Same Count As Associators?",1)
        pyunit.failUnlessEqual(len(refNames), len(assocNames), 
                             "Number of objects returned by ReferenceNames(%d) "
                             "and AssociatorNames(%d) doesn't match" 
                             %(len(refNames), len(assocNames)))
        _print("Same Objects?",1)
        for rname in refNames:
            rname.host = None
        for ref in refs:
            ref.path.host = None
            if ref.path not in refNames:
                pyunit.fail("References returned an object %s not returned "
                            "by ReferenceNames" %ref.path)
        _print("Same Objects As Associators?",1)
        for rname in refNames:
#rname[revAssocRoleName].host=getfqdn()
            if rname[revAssocRoleName] not in assocNames:
                pyunit.fail("ReferenceNames returned an object \n[%s]\n not returned "
                            "by AssociatorNames \n%s" %(rname[revAssocRoleName], assocNames))

    _print("Leaving AssociationSanityCheck")


def instance_sanity_check(pyunit, 
                         conn, 
                         className, 
                         verbose = False):
    global _verbose
    _verbose = verbose
    reqProps=[]

    _print("")
    _print("")

    #get the class
    klass=conn.GetClass(className, LocalOnly=False)
    #get the 'required' properties
    for propName,prop in klass.properties.items():
        if 'Required' in prop.qualifiers:
            if prop.qualifiers['Required'].value is True:
                reqProps.append(propName)

    _print("Entering InstanceSanityCheck.\n\t\tClassName: %s\n\t\t"
           %(className),0)

    _print("")
    _print("Test enumInstances / enumInstanceNames for sanity (same # objects, etc)",0)
    #get assoc name count
    _print("Enumerating instance names of %s" %className, 1)
    instNames = conn.EnumerateInstanceNames(className)
    _print("Resulted in %d objects" %len(instNames),2)

    #get assoc instance count
    _print("Enumerating instances of %s" %className, 1)
    insts = conn.EnumerateInstances(className)
    _print("Resulted in %d objects" %len(insts),2)

    #compare instName count with inst count
    _print("Same Count?",1)
    pyunit.failUnlessEqual(len(instNames), len(insts), 
                     "Number of instance names (%d) and instances (%d) "
                     "returned don't match." %(len(instNames), len(insts)))

    # make sure names match... 
    # inst.path in instances should be found in instNames
    _print("Same Objects?",1)
    for inst in insts:
        if inst.path not in instNames:
            pyunit.fail("EnumInstances returned an instance not found in "
                        "results from EnumInstanceNames")

    _print("Objects Legit?",1)
    #make sure objects are legit, etc
    for instName in instNames:
        _print("legit for %s" %instName,2)
        # try get instance
        try:
            _print("Checking 'GetInstance' on the instName %s" %str(instName),3)
            testInst = conn.GetInstance(instName)
            if testInst.path != instName:
                pyunit.fail("In GetInstance, got an instance, but not the "
                            "one I was expecting")
        except pywbem.CIMError, arg:
            pyunit.fail("Got unexected error trying to 'GetInstance' on the "
                        "instName: %s" %arg)

    #all 'required' properties filled in?
    _print("Required properties not empty?",1)
    for inst in insts:
        _print("Checking instance: %s" %inst.path,2)
        for propName in reqProps:
            _print("Checking required property: %s" %propName,3)
            if inst[propName] is None or inst[propName] == "":
                pyunit.fail("Retrieved an instance with empty required "
                            "property [%s].  InstanceName: %s " %(propName, inst.path))

    _print("Leaving InstanceSanityCheck")





#!/usr/bin/env python

""" Test script to validate the correctness of
association handling in a CIM provider interface.

Exercises the classes defined in TestAssoc.mof
and the corresponding provider in TestAssocProvider.py.
"""

import unittest
import optparse
import pywbem

from lib import wbem_connection
conn = None
   
class TestNullArray(unittest.TestCase):

    def _dbgPrint(self, msg=''):
        if self._verbose:
            if len(msg):
                print('\t -- %s --' % msg)
            else:
                print('')

    def setUp(self):
        unittest.TestCase.setUp(self)
        self._verbose = _globalVerbose
        self._conn = conn
        self._dbgPrint()
        
    def tearDown(self):
        unittest.TestCase.tearDown(self)

    def test_null_array(self):
        """
            Run EnumInstances on the test class and check, that
            1) all instances are returned.
            2) no error is returned.

            The main purpose of this test is to check that NULL arrays can
            be returned from Pegasus CIMOM. It used to return
            CIMError: (1, '61') instead.
        """
        self._dbgPrint('Reading Test_NullArray instances.')
        instances = self._conn.EnumerateInstances('Test_NullArray')
        self.assertEquals(len(instances), 3)
        self._dbgPrint('Instances are OK.')

if __name__ == '__main__':
    parser = optparse.OptionParser()
    wbem_connection.getWBEMConnParserOptions(parser)
    parser.add_option('--level',
            '-l',
            action='store',
            type='int',
            dest='dbglevel',
            help='Indicate the level of debugging statements to display (default=2)',
            default=2)
    parser.add_option('--verbose', '', action='store_true', default=False,
            help='Show verbose output')
    options, arguments = parser.parse_args()
    
    _globalVerbose = options.verbose

    conn = wbem_connection.WBEMConnFromOptions(parser)
    suite = unittest.makeSuite(TestNullArray)
    unittest.TextTestRunner(verbosity=options.dbglevel).run(suite)



"""Python Provider for Test_Except

Instruments the CIM class Test_Except

"""

import pywbem
from pywbem.cim_provider2 import CIMProvider2

class Test_Except(CIMProvider2):
    """Instrument the CIM class Test_Except 

    Instance provider used to test exceptions by causing them
    
    """

    def __init__ (self, env):
        logger = env.get_logger()
        logger.log_debug('Initializing provider %s from %s' \
                % (self.__class__.__name__, __file__))

    def get_instance(self, env, model):
        """Return an instance.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        model -- A template of the pywbem.CIMInstance to be returned.  The 
            key properties are set on this instance to correspond to the 
            instanceName that was requested.  The properties of the model
            are already filtered according to the PropertyList from the 
            request.  Only properties present in the model need to be
            given values.  If you prefer, you can set all of the 
            values, and the instance will be filtered for you. 

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized 
            or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the CIM Class does exist, but the requested CIM 
            Instance does not exist in the specified namespace)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """
        
        logger = env.get_logger()
        logger.log_debug('Entering %s.get_instance()' \
                % self.__class__.__name__)
        

        # TODO fetch system resource matching the following keys:
        #   model['id']

        #model['count'] = pywbem.Uint32() # TODO 
        #model['counts'] = [pywbem.Uint32(),] # TODO 
        #model['flag'] = bool() # TODO 
        #model['flags'] = [bool(),] # TODO 
        #model['message'] = '' # TODO 
        #model['messages'] = ['',] # TODO 
        return model

    def enum_instances(self, env, model, keys_only):
        """Enumerate instances.

        The WBEM operations EnumerateInstances and EnumerateInstanceNames
        are both mapped to this method. 
        This method is a python generator

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        model -- A template of the pywbem.CIMInstances to be generated.  
            The properties of the model are already filtered according to 
            the PropertyList from the request.  Only properties present in 
            the model need to be given values.  If you prefer, you can 
            always set all of the values, and the instance will be filtered 
            for you. 
        keys_only -- A boolean.  True if only the key properties should be
            set on the generated instances.

        Possible Errors:
        CIM_ERR_FAILED (some other unspecified error occurred)

        """

        logger = env.get_logger()
        logger.log_debug('Entering %s.enum_instances()' \
                % self.__class__.__name__)
                
        # Prime model.path with knowledge of the keys, so key values on
        # the CIMInstanceName (model.path) will automatically be set when
        # we set property values on the model. 
        model.path.update({'id': None})
        
        model['id'] = 'RED'

        # ATTN: this next line trashes memory in sfcb (invalid write).
        #model['id'] = pywbem.Uint32(99)

        # ATTN: this next line trashes memory and client hangs forever.
        #model['id'] = ['RED', 'GREEN', 'BLUE' ]

        model['count'] = pywbem.Uint32(99)
        model['flag'] = bool(0)
        model['message'] = 'hello world'
        # ATTN: this next line trashes memory and client hangs forever.
        #model['message'] = ['RED', 'GREEN', 'BLUE' ]
        model['counts'] = [pywbem.Uint32(10), pywbem.Uint32(20)]
        model['flags'] = [bool(1), bool(2), bool(3)]
        model['messages'] = ['RED', 'GREEN', 'BLUE' ]

        # ATTN: sfcb accepts this and sends it to the client.
        #model['illegal property name'] = 'oops'
        yield model

    def set_instance(self, env, instance, modify_existing):
        """Return a newly created or modified instance.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        instance -- The new pywbem.CIMInstance.  If modifying an existing 
            instance, the properties on this instance have been filtered by 
            the PropertyList from the request.
        modify_existing -- True if ModifyInstance, False if CreateInstance

        Return the new instance.  The keys must be set on the new instance. 

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_NOT_SUPPORTED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized 
            or otherwise incorrect parameters)
        CIM_ERR_ALREADY_EXISTS (the CIM Instance already exists -- only 
            valid if modify_existing is False, indicating that the operation
            was CreateInstance)
        CIM_ERR_NOT_FOUND (the CIM Instance does not exist -- only valid 
            if modify_existing is True, indicating that the operation
            was ModifyInstance)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """

        logger = env.get_logger()
        logger.log_debug('Entering %s.set_instance()' \
                % self.__class__.__name__)
        # TODO create or modify the instance
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED) # Remove to implement
        return instance

    def delete_instance(self, env, instance_name):
        """Delete an instance.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        instance_name -- A pywbem.CIMInstanceName specifying the instance 
            to delete.

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_NOT_SUPPORTED
        CIM_ERR_INVALID_NAMESPACE
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized 
            or otherwise incorrect parameters)
        CIM_ERR_INVALID_CLASS (the CIM Class does not exist in the specified 
            namespace)
        CIM_ERR_NOT_FOUND (the CIM Class does exist, but the requested CIM 
            Instance does not exist in the specified namespace)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """ 

        logger = env.get_logger()
        logger.log_debug('Entering %s.delete_instance()' \
                % self.__class__.__name__)

        # TODO delete the resource
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED) # Remove to implement
        
## end of class Test_ExceptProvider
    
## get_providers() for associating CIM Class Name to python provider class name
    
def get_providers(env): 
    test_except_prov = Test_Except(env)  
    return {'Test_Except': test_except_prov} 



#!/usr/bin/env python

import pywbem
from subprocess import call, Popen, PIPE
from time import sleep
import sys

conn = pywbem.SFCBUDSConnection()
conn.default_namespace = 'root/interop'

cnt = 0
while True:
    call('/etc/init.d/sfcb start', shell=True)
    for i in xrange(0,20):
        sleep(1)
        try:
            cnames = conn.EnumerateClassNames()
            break;
        except:
            exc_class, exc, tb = sys.exc_info()
        sys.stdout.write('.'); sys.stdout.flush()
        sleep(1)
    else:
        raise exc_class, exc, tb 
    print '\nfetched %s classes' % len(cnames)
    print cnt
    cnt+= 1
    call('/etc/init.d/sfcb stop', shell=True)
    for i in xrange(0,20):
        po = Popen('ps ax | grep sfcbd | grep -v grep', shell=True, stdout=PIPE)
        procs = po.stdout.read()
        if po.wait() != 0:
            break
        sys.stdout.write('.'); sys.stdout.flush()
        sleep(1)
    else:
        print '\nError: not all sfcbd processes stopped'
        print procs
        sys.exit(1)






#!/usr/bin/env python

""" Test script to validate the correctness of
association handling in a CIM provider interface.

Exercises the classes defined in TestAssoc.mof
and the corresponding provider in TestAssocProvider.py.
"""

import unittest
import optparse
import pywbem
import pwd
import grp

from lib import ProviderSanityTest as PST
from lib import wbem_connection
from socket import getfqdn

_globalVerbose = False

conn = None
   
def isObjPathMatch( op1, op2 ):
    op1.host = None
    op2.host = None
    _local_op1_str = str(op1)
    _local_op2_str = str(op2)
    """
    This function is used to manipulate CIMInstanceName objects to
    determine if they are the same.
    In openwbem, for some reason if you take some object, follow
    a reference to a related object, then follow the reference back
    to the original, the reference back has the hostname prepended
    to the beginning of the object path, whereas the original does not.
    So this function considers this possibility in comparing two
    object paths.
    If you want to not accomodate this discrepancy, comment out the
    rest of the code up to but not including the return statement.
    """
    '''
    # removed in favor of ...host = None above
    hostname = '//%s' % getfqdn()
    if -1 == _local_op1_str.find(hostname, 0, len(hostname)):
        _local_op1_str = '%s/%s' % (hostname,str(op1))
    if -1 == _local_op2_str.find(hostname, 0, len(hostname)):
        _local_op2_str = '%s/%s' % (hostname,str(op2))
    '''
    return _local_op1_str == _local_op2_str
    

        

class TestAssociations(unittest.TestCase):

    def _dbgPrint(self, msg=''):
        if self._verbose:
            if len(msg):
                print('\t -- %s --' % msg)
            else:
                print('')

    def setUp(self):
        unittest.TestCase.setUp(self)
        self._verbose = _globalVerbose
        self._conn = conn
        self._dbgPrint()
        
    def tearDown(self):
        unittest.TestCase.tearDown(self)
    
    def test_a_users(self):
        """
        This test validates that the number of TestAssoc_User objects
        reported from the CIMOM is the same as the number of users
        we get directly from the system via pwd.getpwall(), and that
        the users are the same in both sets.
        """
        self._dbgPrint('Checking validity of TestAssoc_User objects.')
        cimom_users = self._conn.EnumerateInstances('TestAssoc_User', LocalOnly=False)
        sys_users = pwd.getpwall()
        self._dbgPrint('Comparing number of TestAssoc_User objects with system users...')
        if len(cimom_users) != len(sys_users):
            self.fail('Different number of users reported from CIMOM than are in system.')
        self._dbgPrint('Number of users matches.')
        self._dbgPrint('Matching every TestAssoc_User to a system user...')
        for cu in cimom_users:
            found = False
            for i in xrange(len(sys_users)):
                if int(cu['UserID']) == sys_users[i][2]:
                    found = True
                    if cu['UserName'] != sys_users[i][0] or \
                            cu['HomeDirectory'] != sys_users[i][5] or \
                            cu['LoginShell'] != sys_users[i][6]:
                        self.fail('%s "%s" %s' % ('User information for user name',
                                cu['UserName'],
                                'from CIMOM does not match system information.'))
                    sys_users.pop(i)
                    break
            if not found:
                self.fail('User name "%s" from CIMOM not found in system.' % cu['UserName'])
        if 0 < len(sys_users):
            self.fail('Not all system users found in list of users from CIMOM.')
        self._dbgPrint('All users match.')
        
    def test_a_groups(self):
        """
        Same as test_a_users but for TestAssoc_Group and system groups.
        """
        self._dbgPrint('Checking validity of TestAssoc_Group objects.')
        cimom_groups = self._conn.EnumerateInstances('TestAssoc_Group', LocalOnly=False)
        sys_groups = grp.getgrall()
        self._dbgPrint('Comparing number of TestAssoc_Group objects with system groups...')
        if len(cimom_groups) != len(sys_groups):
            self.fail('Different number of groups reported from CIMOM than are in system.')
        self._dbgPrint('Number of groups matches.')
        self._dbgPrint('Matching every TestAssoc_Group to a system group...')
        for cg in cimom_groups:
            found = False
            for i in xrange(len(sys_groups)):
                if int(cg['GroupID']) == sys_groups[i][2]:
                    found = True
                    if cg['GroupName'] != sys_groups[i][0]:
                        self.fail('%s "%s" %s' % ('Group information from group name',
                                cg['GroupName'],
                                'from CIMOM does not match system information.'))
                    sys_groups.pop(i)
                    break
            if not found:
                self.fail('Group name "%s" from CIMOM not found in system.' % cg['GroupName'])
        if 0 < len(sys_groups):
            self.fail('Not all system groups found in list of groups from CIMOM.')
        self._dbgPrint('All groups match.')

    def test_b_usergroups(self):
        """
        This test fetches all TestAssoc_User objects from the CIMOM.  For each user object,
        the references for that user object are retrieved.  The test validates that each group
        associated with this user is correct based upon system information, and also that the
        correct primary group is identified.
        """
        self._dbgPrint('Validating TestAssoc_User group references with system information.')
        cimom_users = self._conn.EnumerateInstances('TestAssoc_User', LocalOnly=False)
        for cu in cimom_users:
            self._dbgPrint('Checking group references for user %s' % cu['UserName'])
            pwinfo = pwd.getpwuid(int(int(cu['UserID'])))
            #for ref in refs:
            for ref in self._conn.References(cu.path, ResultClass='TestAssoc_MemberOfGroup'):
                grp = self._conn.GetInstance(ref['Antecedent'], LocalOnly=False)
                if pwinfo[3] == int(grp['GroupID']):
                    if ref['isPrimaryGroup']:
                        self._dbgPrint('Correctly identified primary group %s for user %s' % (grp['GroupName'], cu['UserName']))
                        break
                    else:
                        self.fail('%s "%s" %s "%s" %s' % ('Group',
                                grp['GroupName'],
                                'not designated as primary for user',
                                pwinfo[0],
                                'when it should be.'))
                else:
                    grinfo = grp.getgruid(int(grp['GroupID']))
                    if pwinfo[0] in grinfo[3]:
                        self._dbgPrint('Correctly identified supplemental group %s for user %s' % (grp['GroupName'], cu['UserName']))
                        break
                    else:
                        self.fail('%s "%s" %s "%s" %s' % ('User',
                                pwinfo[0],
                                'not a member of group',
                                ref['GroupName'],
                                'when it should be.'))
        
    def test_b_groupusers(self):
        """
        This test fetches all TestAssoc_Group objects from the CIMOM.  For each group object,
        the references for that group object are retrieved.  The test validates that each user
        associated with this group is correct based upon system information, and also that the
        correct primary groups are identified.
        """
        self._dbgPrint('Validating TestAssoc_Group user references with system information.')
        cimom_groups = self._conn.EnumerateInstances('TestAssoc_Group', LocalOnly=False)
        for cg in cimom_groups:
            refs = self._conn.References(cg.path, ResultClass='TestAssoc_MemberOfGroup')
            grinfo = grp.getgrgid(int(int(cg['GroupID'])))
            for ref in refs:
                usr = self._conn.GetInstance(ref['Dependent'], LocalOnly=False)
                pwinfo = pwd.getpwuid(int(usr['UserID']))
                if pwinfo[0] in grinfo[3]:
                    if ref['isPrimaryGroup']:
                        self.fail('%s "%s" %s "%s" %s' % ('Group',
                                cg['GroupName'],
                                'designated as primary for user',
                                pwinfo[0],
                                'when it should not be.'))
                    else:
                        self._dbgPrint('Correctly identified supplmental group %s for user %s' % (cg['GroupName'], usr['UserName']))
                elif pwinfo[3] == int(cg['GroupID']):
                    if not ref['isPrimaryGroup']:
                        self.fail('%s "%s" %s "%s" %s' % ('Group',
                                cg['GroupName'],
                                'not designated as primary for user',
                                pwinfo[0],
                                'when it should be.'))
                    else:
                        self._dbgPrint('Correctly identified primary group %s for user %s' % (cg['GroupName'], usr['UserName']))
                            
    def test_c_sanitytest(self):
        PST.association_sanity_check(self, self._conn, 'TestAssoc_MemberOfGroup', self._verbose)
        
    def test_d_user_op_references_traversal(self):
        """
        This test fetches all user object paths, follows references for these object paths
        to group object paths, then follows references back, and checks to see that the same
        user object path can be found and that it fetches the same references as the original.
        """
        self._dbgPrint('Validating user object path references traversal.')
        for uop in self._conn.EnumerateInstanceNames('TestAssoc_User'):
            self._dbgPrint('Getting user references for object path %s' % uop )
            urefs = self._conn.References(uop, ResultClass='TestAssoc_MemberOfGroup')
            for uref in urefs:
                uopFound = False
                self._dbgPrint('Getting group references for object path %s' % uref['Antecedent'])
                for gref in self._conn.References(uref['Antecedent'], ResultClass='TestAssoc_MemberOfGroup'):
                    if uop == gref['Dependent']:
                        self._dbgPrint('Found matching user object path.')
                        uopFound = True
                        for uref2 in self._conn.References(gref['Dependent'], ResultClass='TestAssoc_MemberOfGroup'):
                            if not uref2 in urefs:
                                userobj = self._conn.GetInstance(uop, LocalOnly=False)
                                self.fail('Failed to retrieve identical references for user "%s"' % userobj['UserName'])
                        self._dbgPrint('Found matching references for user object path.')
                        break
                if not uopFound:
                    userobj = self._conn.GetInstance(uop, LocalOnly=False)
                    grpobj = self._conn.GetInstance(uref['Antecedent'], LocalOnly=False)
                    self.fail('%s "%s" %s "%s" %s' % ('Reference to group',
                            grpobj['GroupName'],
                            'from user',
                            userobj['UserName'],
                            'doesn\'t return to user.'))
                            
    def test_d_user_op_referencenames_traversal(self):
        """
        Same as test_d_user_op_references_traversal, but using the ReferenceNames call
        instead of the References call.
        """
        self._dbgPrint('Validating user object path reference names traversal.')
        for uop in self._conn.EnumerateInstanceNames('TestAssoc_User'):
            self._dbgPrint('Getting user reference names for object path %s' % uop)
            urefs = self._conn.ReferenceNames(uop, ResultClass='TestAssoc_MemberOfGroup')
            for uref in urefs:
                uopFound = False
                self._dbgPrint('Getting group reference names for object path %s' % uref['Antecedent'])
                for gref in self._conn.ReferenceNames(uref['Antecedent'], ResultClass='TestAssoc_MemberOfGroup'):
                    if uop == gref['Dependent']:
                        self._dbgPrint('Found matching user object path.')
                        uopFound = True
                        for uref2 in self._conn.ReferenceNames(gref['Dependent'], ResultClass='TestAssoc_MemberOfGroup'):
                            if not uref2 in urefs:
                                userobj = self._conn.GetInstance(uop, LocalOnly=False)
                                self.fail('Failed to retrieve identical references for user "%s"' % userobj['UserName'])
                        self._dbgPrint('Found matching references for user object path.')
                        break
                if not uopFound:
                    userobj = self._conn.GetInstance(uop, LocalOnly=False)
                    grpobj = self._conn.GetInstance(uref['Antecedent'], LocalOnly=False)
                    self.fail('%s "%s" %s "%s" %s' % ('Reference to group',
                            grpobj['GroupName'],
                            'from user',
                            userobj['UserName'],
                            'doesn\'t return to user.'))
                            
    def test_e_user_references_traversal(self):
        """
        Same as test_d_user_op_references_traversal, but using users instead
        of object paths, i.e. initially calls EnumerateInstances instead of EnumerateInstanceNames.
        """
        self._dbgPrint('Validating user object references traversal.')
        for userobj in self._conn.EnumerateInstances('TestAssoc_User', LocalOnly=False):
            self._dbgPrint('Getting user references for user %s' % userobj['UserName'])
            urefs = self._conn.References(userobj.path, ResultClass='TestAssoc_MemberOfGroup')
            for uref in urefs:
                userobjFound = False
                self._dbgPrint('Getting group references for user object path %s' % uref['Antecedent'])
                for gref in self._conn.References(uref['Antecedent'], ResultClass='TestAssoc_MemberOfGroup'):
                    userobj2 = self._conn.GetInstance(gref['Dependent'], LocalOnly=False)
                    if userobj == userobj2:
                        self._dbgPrint('Found matching user object.')
                        userobjFound = True
                        for uref2 in self._conn.References(gref['Dependent'], ResultClass='TestAssoc_MemberOfGroup'):
                            if not uref2 in urefs:
                                self.fail('Failed to retrieve identical references for user "%s"' % userobj['UserName'])
                        self._dbgPrint('Found matching references for user object.')
                        break
                if not userobjFound:
                    grpobj = self._conn.GetInstance(uref['Antecedent'], LocalOnly=False)
                    self.fail('%s "%s" %s "%s" %s' % ('Reference to group',
                            grpobj['GroupName'],
                            'from user',
                            userobj['UserName'],
                            'doesn\'t return to user.'))
                            
    def test_e_user_referencenames_traversal(self):
        """
        Same as test_e_user_references_traversal, but calling ReferenceNames instead
        of References.
        """
        self._dbgPrint('Validating user object reference names traversal.')
        for userobj in self._conn.EnumerateInstances('TestAssoc_User', LocalOnly=False):
            self._dbgPrint('Getting user reference names for user %s' % userobj['UserName'])
            urefs = self._conn.ReferenceNames(userobj.path, ResultClass='TestAssoc_MemberOfGroup')
            for uref in urefs:
                userobjFound = False
                self._dbgPrint('Getting group references for user object path %s' % uref['Antecedent'])
                for gref in self._conn.ReferenceNames(uref['Antecedent'], ResultClass='TestAssoc_MemberOfGroup'):
                    userobj2 = self._conn.GetInstance(gref['Dependent'], LocalOnly=False)
                    if userobj == userobj2:
                        self._dbgPrint('Found matching user object.')
                        userobjFound = True
                        for uref2 in self._conn.ReferenceNames(gref['Dependent'], ResultClass='TestAssoc_MemberOfGroup'):
                            if not uref2 in urefs:
                                self.fail('Failed to retrieve identical references for user "%s"' % userobj['UserName'])
                        self._dbgPrint('Found matching reference names for user object.')
                        break
                if not userobjFound:
                    grpobj = self._conn.GetInstance(uref['Antecedent'], LocalOnly=False)
                    self.fail('%s "%s" %s "%s" %s' % ('Reference to group',
                            grpobj['GroupName'],
                            'from user',
                            userobj['UserName'],
                            'doesn\'t return to user.'))
                
    def test_f_user_op_associators_traversal(self):
        """
        Same as test_d_user_op_references_traversal but calls Associators
        instead of References; thus we skip looking up the group object
        for each reference Antecedent.
        """
        self._dbgPrint('Validating user object path associators traversal.')
        for uop in self._conn.EnumerateInstanceNames('TestAssoc_User'):
            self._dbgPrint('Getting associated groups for user object path %s' % uop)
            for grp in self._conn.Associators(uop, AssocClass='TestAssoc_MemberOfGroup'):
                usrFound = False
                self._dbgPrint('Getting associated users for group object path %s' % grp.path)
                for usr in self._conn.Associators(grp.path, AssocClass='TestAssoc_MemberOfGroup'):
                    if isObjPathMatch( usr.path, uop ):
                        usrFound = True
                        self._dbgPrint('Found matching user object.')
                        if not grp in self._conn.Associators(usr.path, AssocClass='TestAssoc_MemberOfGroup'):
                            self.fail('Failed to retrieve identical associators for user "%s"' % usr['UserName'])
                        self._dbgPrint('Found matching associators for user %s' % usr['UserName'])
                        break
                if not usrFound:
                    self.fail('%s "%s" %s "%s" %s' % ('Associator to group',
                            grp['GroupName'],
                            'from user',
                            usr['UserName'],
                            'doesn\'t return to user.'))
    
    def test_f_user_op_associatornames_traversal(self):
        """
        Same as test_f_user_op_associators_traversal, but calls
        AssociatorNames instead of Associators.
        """
        self._dbgPrint('Validating user object path associator names traversal.')
        for uop in self._conn.EnumerateInstanceNames('TestAssoc_User'):
            self._dbgPrint('Getting associated group names for user object path %s' % uop)
            for grp in self._conn.AssociatorNames(uop, AssocClass='TestAssoc_MemberOfGroup'):
                usrFound = False
                self._dbgPrint('Getting associated user names for group object path %s' % grp)
                for usr in self._conn.AssociatorNames(grp, AssocClass='TestAssoc_MemberOfGroup'):
                    if isObjPathMatch( usr, uop ):
                        usrFound = True
                        self._dbgPrint('Found matching user object path.')
                        if not grp in self._conn.AssociatorNames(usr, AssocClass='TestAssoc_MemberOfGroup'):
                            userobj = self._conn.GetInstance(usr, LocalOnly=False)
                            self.fail('Failed to retrieve identical associators for user "%s"' % userobj['UserName'])
                        self._dbgPrint('Found matching associator names for user object path %s' % uop)
                        break
                if not usrFound:
                    userobj = self._conn.GetInstance(usr, LocalOnly=False)
                    self.fail('%s "%s" %s "%s" %s' % ('Associator to group',
                            grp['GroupName'],
                            'from user',
                            userobj['UserName'],
                            'doesn\'t return to user.'))
            
    def test_g_user_associators_traversal(self):
        """
        Same as test_f_user_op_associators_traversal, but uses user objects
        instead of user object paths, i.e. initially calls EnumerateInstances
        instead of EnumerateInstanceNames.
        Performs a deeper check of user object equivalence.
        """
        self._dbgPrint('Validating user associators traversal.')
        for usr in self._conn.EnumerateInstances('TestAssoc_User', LocalOnly=False):
            self._dbgPrint('Getting associated groups for user %s' % usr['UserName'])
            for grp in self._conn.Associators(usr.path, AssocClass='TestAssoc_MemberOfGroup'):
                usrFound = False
                self._dbgPrint('Getting associated users for group %s' % grp['GroupName'])
                for usr2 in self._conn.Associators(grp.path, AssocClass='TestAssoc_MemberOfGroup'):
                    if isObjPathMatch( usr2.path, usr.path ) and \
                            usr2['UserName'] == usr['UserName'] and \
                            usr2['UserID'] == usr['UserID']:
                        usrFound = True
                        self._dbgPrint('Found matching user.')
                        if not grp in self._conn.Associators(usr2.path, AssocClass='TestAssoc_MemberOfGroup'):
                            self.fail('Failed to retrieve identical associators for user "%s"' % usr2['UserName'])
                        self._dbgPrint('Found matching associated objects for user %s' % usr['UserName'])
                        break
                if not usrFound:
                    self.fail('%s "%s" %s "%s" %s' % ('Associator to group',
                            grp['GroupName'],
                            'from user',
                            usr['UserName'],
                            'doesn\'t return to user.'))
        
    def test_g_user_associatornames_traversal(self):
        """
        Same as test_g_user_associators_traversal but calls AssociatorNames instead of Associators.
        """
        self._dbgPrint('Validating user associator names traversal.')
        for usr in self._conn.EnumerateInstances('TestAssoc_User', LocalOnly=False):
            self._dbgPrint('Getting associated group names for user %s' % usr['UserName'])
            for grp in self._conn.AssociatorNames(usr.path, AssocClass='TestAssoc_MemberOfGroup'):
                usrFound = False
                self._dbgPrint('Getting associated user names for group object path %s' % grp)
                for usr2 in self._conn.AssociatorNames(grp, AssocClass='TestAssoc_MemberOfGroup'):
                    if isObjPathMatch( usr2, usr.path ):
                        usrFound = True
                        self._dbgPrint('Found matching user.')
                        if not grp in self._conn.AssociatorNames(usr2, AssocClass='TestAssoc_MemberOfGroup'):
                            userobj = self._conn.GetInstance(usr2, LocalOnly=False)
                            self.fail('Failed to retrieve identical associators for user "%s"' % userobj['UserName'])
                        self._dbgPrint('Found matching associated group object paths for user %s' % usr['UserName'])
                        break
                if not usrFound:
                    self.fail('%s "%s" %s "%s" %s' % ('Associator to group',
                            grp['GroupName'],
                            'from user',
                            usr['UserName'],
                            'doesn\'t return to user.'))
            
    def test_h_group_op_references_traversal(self):
        """
        This test fetches all group object paths, follows references for these object paths
        to user object paths, then follows references back, and checks to see that the same
        group object path can be found and that it fetches the same references as the original.
        """
        self._dbgPrint('Validating group object path references traversal.')
        for gop in self._conn.EnumerateInstanceNames('TestAssoc_Group'):
            self._dbgPrint('Getting group references for object path %s' % gop)
            grefs = self._conn.References(gop, ResultClass='TestAssoc_MemberOfGroup')
            for gref in grefs:
                gopFound = False
                self._dbgPrint('Getting user references for object path %s' % gref['Dependent'])
                for uref in self._conn.References(gref['Dependent'], ResultClass='TestAssoc_MemberOfGroup'):
                    if gop == uref['Antecedent']:
                        self._dbgPrint('Found matching group object path.')
                        gopFound = True
                        for gref2 in self._conn.References(uref['Antecedent'], ResultClass='TestAssoc_MemberOfGroup'):
                            if not gref2 in grefs:
                                grpobj = self._conn.GetInstance(gop, LocalOnly=False)
                                self.fail('Failed to retrieve identical references for group "%s"' % grpobj['GroupName'])
                        self._dbgPrint('Found matching references for group object path.')
                        break
                if not gopFound:
                    grpobj = self._conn.GetInstance(gop, LocalOnly=False)
                    userobj = self._conn.GetInstance(gref['Dependent'], LocalOnly=False)
                    self.fail('%s "%s" %s "%s" %s' % ('Reference to user',
                            userobj['UserName'],
                            'from group',
                            grpobj['GroupName'],
                            'doesn\'t return to group.'))
        
    def test_h_group_op_referencenames_traversal(self):
        """
        Same as test_h_group_op_references_traversal but calls
        ReferenceNames instead of References.
        """
        self._dbgPrint('Validating group object path reference names traversal.')
        for gop in self._conn.EnumerateInstanceNames('TestAssoc_Group'):
            self._dbgPrint('Getting group reference names for object path %s' % gop)
            grefs = self._conn.ReferenceNames(gop, ResultClass='TestAssoc_MemberOfGroup')
            for gref in grefs:
                gopFound = False
                self._dbgPrint('Getting user reference names for object path %s' % gref['Dependent'])
                for uref in self._conn.ReferenceNames(gref['Dependent'], ResultClass='TestAssoc_MemberOfGroup'):
                    if gop == uref['Antecedent']:
                        self._dbgPrint('Found matching group object path.')
                        gopFound = True
                        for gref2 in self._conn.ReferenceNames(uref['Antecedent'], ResultClass='TestAssoc_MemberOfGroup'):
                            if not gref2 in grefs:
                                grpobj = self._conn.GetInstance(gop, LocalOnly=False)
                                self.fail('Failed to retrieve identical references for group "%s"' % grpobj['GroupName'])
                        self._dbgPrint('Found matching reference names for group object path.')    
                        break
                if not gopFound:
                    grpobj = self._conn.GetInstance(gop, LocalOnly=False)
                    userobj = self._conn.GetInstance(gref['Dependent'], LocalOnly=False)
                    self.fail('%s "%s" %s "%s" %s' % ('Reference to user',
                            userobj['UserName'],
                            'from group',
                            grpobj['GroupName'],
                            'doesn\'t return to group.'))
    
    def test_i_group_references_traversal(self):
        """
        Same as test_h_group_op_references_traversal but uses group
        objects instead of object paths, i.e. initially calls EnumerateInstances
        instead of EnumerateInstanceNames.
        """
        self._dbgPrint('Validating group references traversal.')
        for grp in self._conn.EnumerateInstances('TestAssoc_Group', LocalOnly=False):
            self._dbgPrint('Getting group references for group %s' % grp['GroupName'])
            grefs = self._conn.References(grp.path, ResultClass='TestAssoc_MemberOfGroup')
            for gref in grefs:
                grpFound = False
                self._dbgPrint('Getting user references for object path %s' % gref['Dependent'])
                for uref in self._conn.References(gref['Dependent'], ResultClass='TestAssoc_MemberOfGroup'):
                    if grp.path == uref['Antecedent']:
                        self._dbgPrint('Found matching group.')
                        grpFound = True
                        for gref2 in self._conn.References(uref['Antecedent'], ResultClass='TestAssoc_MemberOfGroup'):
                            if not gref2 in grefs:
                                self.fail('Failed to retrieve identical references for group "%s"' % grp['GroupName'])
                        self._dbgPrint('Found matching references for group.')
                        break
                if not grpFound:
                    userobj = self._conn.GetInstance(gref['Dependent'], LocalOnly=False)
                    self.fail('%s "%s" %s "%s" %s' % ('Reference to user',
                            userobj['UserName'],
                            'from group',
                            grp['GroupName'],
                            'doesn\'t return to group.'))
        
    def test_i_group_referencenames_traversal(self):
        """
        Same as test_i_group_references_traversal but calls
        ReferenceNames instead of References.
        """
        self._dbgPrint('Validating group reference names traversal.')
        for grp in self._conn.EnumerateInstances('TestAssoc_Group', LocalOnly=False):
            self._dbgPrint('Getting group reference names for group %s' % grp['GroupName'])
            grefs = self._conn.ReferenceNames(grp.path, ResultClass='TestAssoc_MemberOfGroup')
            for gref in grefs:
                grpFound = False
                self._dbgPrint('Getting user reference names for object path %s' % gref['Dependent'])
                for uref in self._conn.ReferenceNames(gref['Dependent'], ResultClass='TestAssoc_MemberOfGroup'):
                    if grp.path == uref['Antecedent']:
                        self._dbgPrint('Found matching group.')
                        grpFound = True
                        for gref2 in self._conn.ReferenceNames(uref['Antecedent'], ResultClass='TestAssoc_MemberOfGroup'):
                            if not gref2 in grefs:
                                self.fail('Failed to retrieve identical references for group "%s"' % grp['GroupName'])
                        self._dbgPrint('Found matching reference names for group.')
                        break
                if not grpFound:
                    userobj = self._conn.GetInstance(gref['Dependent'], LocalOnly=False)
                    self.fail('%s "%s" %s "%s" %s' % ('Reference to user',
                            userobj['UserName'],
                            'from group',
                            grp['GroupName'],
                            'doesn\'t return to group.'))
       
    def test_j_group_op_associators_traversal(self):
        """
        Same as test_h_group_op_references_traversal but
        calls Associators instead of References.
        """
        self._dbgPrint('Validating group object path associators traversal.')
        for gop in self._conn.EnumerateInstanceNames('TestAssoc_Group'):
            self._dbgPrint('Getting associated users for group object path %s' % gop)
            for usr in self._conn.Associators(gop, AssocClass='TestAssoc_MemberOfGroup'):
                grpFound = False
                self._dbgPrint('Getting associated groups for user %s' % usr['UserName'])
                for grp in self._conn.Associators(usr.path, AssocClass='TestAssoc_MemberOfGroup'):
                    if isObjPathMatch( grp.path, gop ):
                        grpFound = True
                        self._dbgPrint('Found matching group object path.')
                        if not usr in self._conn.Associators(grp.path, AssocClass='TestAssoc_MemberOfGroup'):
                            self.fail('Failed to retrieve identical associators for group "%s"' % grp['GroupName'])
                        self._dbgPrint('Found matching associators for group object path.')
                        break
                if not grpFound:
                    self.fail('%s "%s" %s "%s" %s' % ('Associator to user',
                            usr['UserName'],
                            'from group',
                            grp['GroupName'],
                            'doesn\'t return to group.'))
        
    def test_j_group_op_associatornames_traversal(self):
        """
        Same as test_j_group_op_associators_traversal but
        calls AssociatorNames instead of Associators.
        """
        self._dbgPrint('Validating group object path associator names traversal.')
        for gop in self._conn.EnumerateInstanceNames('TestAssoc_Group'):
            self._dbgPrint('Getting associated user names for group object path %s' % gop)
            for usr in self._conn.AssociatorNames(gop, AssocClass='TestAssoc_MemberOfGroup'):
                grpFound = False
                self._dbgPrint('Getting associated group names for user object path %s' % usr)
                for grp in self._conn.AssociatorNames(usr, AssocClass='TestAssoc_MemberOfGroup'):
                    if isObjPathMatch( grp, gop ):
                        grpFound = True
                        self._dbgPrint('Found matching group object path.')
                        if not usr in self._conn.AssociatorNames(grp, AssocClass='TestAssoc_MemberOfGroup'):
                            grpobj = self._conn.GetInstance(grp, LocalOnly=False)
                            self.fail('Failed to retrieve identical associators for group "%s"' % grpobj['GroupName'])
                        self._dbgPrint('Found matching associator names for group object path.')
                        break
                if not grpFound:
                    usrobj = self._conn.GetInstance(usr, LocalOnly=False)
                    grpobj = self._conn.GetInstance(grp, LocalOnly=False)
                    self.fail('%s "%s" %s "%s" %s' % ('Associator to user',
                            usrobj['UserName'],
                            'from group',
                            grpobj['GroupName'],
                            'doesn\'t return to group.'))
            
    def test_k_group_associators_traversal(self):
        """
        Same as test_j_group_op_associators_traversal but
        uses group objects instead of object paths, i.e. originally
        calls EnumerateInstances instead of EnumerateInstanceNames.
        Performs a deeper object comparison of the two groups.
        """
        self._dbgPrint('Validating group associators traversal.')
        for grp in self._conn.EnumerateInstances('TestAssoc_Group', LocalOnly=False):
            self._dbgPrint('Getting associated users for group %s' % grp['GroupName'])
            for usr in self._conn.Associators(grp.path, AssocClass='TestAssoc_MemberOfGroup'):
                grpFound = False
                self._dbgPrint('Getting associated groups for user %s' % usr['UserName'])
                for grp2 in self._conn.Associators(usr.path, AssocClass='TestAssoc_MemberOfGroup'):
                    if isObjPathMatch( grp2.path, grp.path ) and \
                            grp2['GroupID'] == grp['GroupID'] and \
                            grp2['GroupName'] == grp['GroupName']:
                        grpFound = True
                        self._dbgPrint('Found matching group.')
                        if not usr in self._conn.Associators(grp2.path, AssocClass='TestAssoc_MemberOfGroup'):
                            self.fail('Failed to retrieve identical associators for group "%s"' % grp2['GroupName'])
                        self._dbgPrint('Found matching associated users for group.')
                        break
                if not grpFound:
                    self.fail('%s "%s" %s "%s" %s' % ('Associator to user',
                            usr['UserName'],
                            'from group',
                            grp['GroupName'],
                            'doesn\'t return to group.'))
        
    def test_k_group_associatornames_traversal(self):
        """
        Same as test_k_group_associators_traversal but calls
        AssociatorNames instead of Associators.
        """
        self._dbgPrint('Validating group associator names traversal.')
        for grp in self._conn.EnumerateInstances('TestAssoc_Group', LocalOnly=False):
            self._dbgPrint('Getting associated user names for group %s' % grp['GroupName'])
            for uop in self._conn.AssociatorNames(grp.path, AssocClass='TestAssoc_MemberOfGroup'):
                grpFound = False
                self._dbgPrint('Getting associated group names for user object path %s' % uop)
                for gop in self._conn.AssociatorNames(uop, AssocClass='TestAssoc_MemberOfGroup'):
                    if isObjPathMatch( gop, grp.path ):
                        grpFound = True
                        self._dbgPrint('Found matching group.')
                        if not uop in self._conn.AssociatorNames(gop, AssocClass='TestAssoc_MemberOfGroup'):
                            grpobj = self._conn.GetInstance(gop, LocalOnly=False)
                            self.fail('Failed to retrieve identical associators for group "%s"' % grpobj['GroupName'])
                        self._dbgPrint('Found matching associator names for group.')
                        break
                if not grpFound:
                    usrobj = self._conn.GetInstance(uop, LocalOnly=False)
                    self.fail('%s "%s" %s "%s" %s' % ('Associator to user',
                            usrobj['UserName'],
                            'from group',
                            grp['GroupName'],
                            'doesn\'t return to group.'))
       
    def test_l_user_primarygroup_references_traversal(self):
        """
        This test fetches all TestAssoc_User objects, and for each one, it
        searches the References for the reference that is designated as the
        one to the primary group.  Then it follows that reference to the group
        on the other side, and searches all the references on that side for
        primary group designations, and makes sure one of them points to
        the original user object.
        Note that while a user can only have one primary group, any group may
        be primary for a number of users.
        """
        self._dbgPrint('Validating User->Primary Group->User references traversal.')
        for usr in self._conn.EnumerateInstances('TestAssoc_User', LocalOnly=False):
            self._dbgPrint('Finding primary group for user %s' % usr['UserName'])
            for ref in self._conn.References(usr.path, ResultClass='TestAssoc_MemberOfGroup'):
                if ref['isPrimaryGroup']:
                    usrFound = False
                    self._dbgPrint('Found primary group, object path is %s' % ref['Antecedent'])
                    for ref2 in self._conn.References(ref['Antecedent'], ResultClass='TestAssoc_MemberOfGroup'):
                        if ref2['isPrimaryGroup']:
                            if ref2['Dependent'] == usr.path:
                                usrFound = True
                                break
                    if not usrFound:
                        self.fail('User "%s" primary group does not indicate user as primary.' % usr['UserName'])
                    self._dbgPrint('Retraced primary group reference back to original user.')
                    break
        
    def test_l_user_primarygroup_associators_traversal(self):
        """
        This test is similar to test_l_user_primarygroup_references_traversal
        but uses Associators instead of References.  This means it cannot directly
        check that the relationship is for the primary group using the CIMOM.  So
        what this test does instead is use system information to identify the group
        which *should* be the primary group, then it constructs a CIMInstanceName using
        the user object path as Dependent and the group object path as Antecedent.
        Then it fetches a TestAssoc_MemberOfGroup CIMInstance using this name, and
        verifies that an instance is retrieved and that it is designated as primary.
        """
        self._dbgPrint('Validating User->Primary Group->User associators traversal.')
        for usr in self._conn.EnumerateInstances('TestAssoc_User', LocalOnly=False):
            self._dbgPrint('Finding primary group for user %s' % usr['UserName'])
            grp = None
            for g in self._conn.Associators(usr.path, AssocClass='TestAssoc_MemberOfGroup'):
                if int(g['GroupID']) == pwd.getpwuid(int(usr['UserID']))[3]:
                    grp = g
                    break
            if grp is None:
                self.fail('Couldn\'t find group with group id that matches user "%s" group id.' % usr['UserName'])
            self._dbgPrint('Found primary group candidate %s for user %s' % (grp['GroupName'], usr['UserName']))
            refname = pywbem.CIMInstanceName('TestAssoc_MemberOfGroup',
                    namespace='root/cimv2',
                    keybindings={
                        'Dependent':usr.path,
                        'Antecedent':grp.path})
            ref = self._conn.GetInstance(refname, LocalOnly=False)
            if not ref:
                self.fail('Failed to retrieve reference instance from known associator object paths.')
            if not ref['isPrimaryGroup']:
                self.fail('%s "%s" %s "%s" %s' % ('Reference does not indicate primary group for user',
                        usr['UserName'],
                        'and group',
                        grp['GroupName'],
                        'when it should.'))
            self._dbgPrint('Successfully validated user/group relationship.')
                
    def test_m_group_primarygroup_references_traversal(self):
        """
        This test fetches all TestAssoc_Group objects, and checks every reference that
        indicates a primary group reference.  For every user on the other side of the
        reference, the test fetches references for that user, looks up the primary group
        reference, and makes sure it refers back to the original group.
        """
        self._dbgPrint('Validating Group->Primary Group->Group references traversal.')
        for grp in self._conn.EnumerateInstances('TestAssoc_Group', LocalOnly=False):
            self._dbgPrint('Getting primary group references for group %s' % grp['GroupName'])
            for ref in self._conn.References(grp.path, ResultClass='TestAssoc_MemberOfGroup'):
                if ref['isPrimaryGroup']:
                    pgFound = False
                    self._dbgPrint('%s %s %s, checking its references...' % (grp['GroupName'],
                            'is a primary group for user object path', ref['Dependent']))
                    for ref2 in self._conn.References(ref['Dependent'], ResultClass='TestAssoc_MemberOfGroup'):
                        if ref2['isPrimaryGroup']:
                            pgFound = True
                            if grp.path != ref2['Antecedent']:
                                self.fail('Discovered primary group not the same as the original for "%s".' % grp['GroupName'])
                            break
                    if not pgFound:
                        usr = self._conn.GetInstance(ref['Dependent'], LocalOnly=False)
                        self.fail('Could not find primary group "%s" for discovered group member "%s".' % (grp['GroupName'], usr['UserName']))
                    self._dbgPrint('Retraced primary group reference back to original group.')
                    break
        
    def test_m_group_primarygroup_associators_traversal(self):
        """
        This test is like test_m_group_primarygroup_references_traversal
        but uses Associators instead of References, so it validates the relationship
        by constructing a TestAssoc_MemberOfGroup object from scratch;
        see test_l_user_primarygroup_associators_traversal for details.
        """
        self._dbgPrint('Validating Group->Primary Group->Group associators traversal.')
        for grp in self._conn.EnumerateInstances('TestAssoc_Group', LocalOnly=False):
            self._dbgPrint('Getting associated users for group %s' % grp['GroupName'])
            for usr in self._conn.Associators(grp.path, AssocClass='TestAssoc_MemberOfGroup'):
                if int(grp['GroupID']) == pwd.getpwuid(int(usr['UserID']))[3]:
                    self._dbgPrint('Found user %s that has group %s as primary; checking reference object.' % (usr['UserName'],
                            grp['GroupName']))
                    refname = pywbem.CIMInstanceName('TestAssoc_MemberOfGroup',
                            namespace='root/cimv2',
                            keybindings={
                                'Dependent':usr.path,
                                'Antecedent':grp.path})
                    ref = self._conn.GetInstance(refname, LocalOnly=False)
                    if not ref:
                        self.fail('Failed to get reference instance for group "%s" and user "%s".' % (grp['GroupName'], usr['UserName']))
                    if not ref['isPrimaryGroup']:
                        self.fail('%s "%s" %s "%s" %s' % ('Reference for group',
                                grp['GroupName'],
                                'and user',
                                usr['UserName'],
                                'not designated primary when it should be.'))
                    self._dbgPrint('Successfully validated group/user relationship.')
                                
    def test_n_user_nonprimarygroup_references_traversal(self):
        """
        This test fetches each TestAssoc_User object and checks references to
        each object that are not designated as being the primary group reference.
        Then with each group it fetches references back to the users, and makes
        sure there is a reference to this user and that it is not designated
        as primary.
        """
        self._dbgPrint('Validating User->Supplemental Group->User references traversal.')
        for usr in self._conn.EnumerateInstances('TestAssoc_User', LocalOnly=False):
            self._dbgPrint('Finding supplemental groups for user %s' % usr['UserName'])
            for ref in self._conn.References(usr.path, ResultClass='TestAssoc_MemberOfGroup'):
                if not ref['isPrimaryGroup']:
                    usrFound = False
                    self._dbgPrint('%s %s for user %s, checking references...' % ('Found supplemental group object path',
                            ref['Antecedent'], usr['UserName']))
                    for ref2 in self._conn.References(ref['Antecedent'], ResultClass='TestAssoc_MemberOfGroup'):
                        if not ref2['isPrimaryGroup']:
                            if ref2['Dependent'] == usr.path:
                                usrFound = True
                                break
                    if not usrFound:
                        self.fail('User "%s" non-primary group does not indicate user as member.' % usr['UserName'])
                    self._dbgPrint('Retraced supplemental group reference back to original user.')
                    break
        
    def test_n_user_nonprimarygroup_associators_traversal(self):
        """
        This test is like test_n_user_nonprimarygroup_references_traversal except
        it uses Associators instead of References.  Like test_l_user_primarygroup_associators_traversal,
        it constructs a TestAssoc_MemberOfGroup reference object path from scratch,
        fetches the instance, and validates that it has the expected properties.
        """
        self._dbgPrint('Validating User->Supplemental Group->User associators traversal.')
        for usr in self._conn.EnumerateInstances('TestAssoc_User', LocalOnly=False):
            self._dbgPrint('Finding supplemental groups for user %s' % usr['UserName'])
            for grp in self._conn.Associators(usr.path, AssocClass='TestAssoc_MemberOfGroup'):
                if int(grp['GroupID']) == pwd.getpwuid(int(usr['UserID']))[3]:
                    continue
                self._dbgPrint('Found supplemental group %s for user %s, validating...' % (grp['GroupName'], usr['UserName']))
                refname = pywbem.CIMInstanceName('TestAssoc_MemberOfGroup',
                        namespace='root/cimv2',
                        keybindings={
                            'Dependent':usr.path,
                            'Antecedent':grp.path})
                ref = self._conn.GetInstance(refname, LocalOnly=False)
                if not ref:
                    self.fail('Failed to retrieve reference instance from known associator object paths.')
                if ref['isPrimaryGroup']:
                    self.fail('%s "%s" %s "%s" %s' % ('Reference does not indicate non-primary group for user',
                            usr['UserName'],
                            'and group',
                            grp['GroupName'],
                            'when it should.'))
                self._dbgPrint('Successfully validated user/group relationship.')
        
    def test_o_group_nonprimarygroup_references_traversal(self):
        """
        This test is like test_n_user_nonprimarygroup_references_traversal but
        it begins with group objects instead of user objects.
        """
        self._dbgPrint('Validating Group->Supplemental Group->Group references traversal.')
        for grp in self._conn.EnumerateInstances('TestAssoc_Group', LocalOnly=False):
            self._dbgPrint('Fetching user references for group %s' % grp['GroupName'])
            for ref in self._conn.References(grp.path, ResultClass='TestAssoc_MemberOfGroup'):
                if not ref['isPrimaryGroup']:
                    grpFound = False
                    self._dbgPrint('Found supplemental user object path %s, checking groups...' % ref['Dependent'])
                    for ref2 in self._conn.References(ref['Dependent'], ResultClass='TestAssoc_MemberOfGroup'):
                        if not ref2['isPrimaryGroup'] and grp.path == ref2['Antecedent']:
                            grpFound = True
                            break
                    if not grpFound:
                        usr = self._conn.GetInstance(ref['Dependent'], LocalOnly=False)
                        self.fail('%s "%s" for discovered group member "%s".' % ('Could not find non-primary group',
                                grp['GroupName'], usr['UserName']))
                    self._dbgPrint('Retraced supplemental group reference back to original group.')
                    break
        
    def test_o_group_nonprimarygroup_associators_traversal(self):
        """
        This test is like test_o_group_nonprimarygroup_references_traversal but
        calls Associators instead of References, constructing the reference object
        path and validating the instance like in test_n_user_nonprimarygroup_associators_traversal.
        """
        self._dbgPrint('Validating Group->Supplemental Group->Group associators traversal.')
        for grp in self._conn.EnumerateInstances('TestAssoc_Group', LocalOnly=False):
            self._dbgPrint('Fetching associated users for group %s' % grp['GroupName'])
            for usr in self._conn.Associators(grp.path, AssocClass='TestAssoc_MemberOfGroup'):
                if int(grp['GroupID']) != pwd.getpwuid(int(usr['UserID']))[3]:
                    self._dbgPrint('%s %s for group %s, validating...' % ('Found candidate supplemental user',
                            usr['UserName'], grp['GroupName']))
                    refname = pywbem.CIMInstanceName('TestAssoc_MemberOfGroup',
                            namespace='root/cimv2',
                            keybindings={
                                'Dependent':usr.path,
                                'Antecedent':grp.path})
                    ref = self._conn.GetInstance(refname, LocalOnly=False)
                    if not ref:
                        self.fail('Failed to get reference instance for group "%s" and user "%s".' % (grp['GroupName'], usr['UserName']))
                    if ref['isPrimaryGroup']:
                        self.fail('%s "%s" %s "%s" %s' % ('Reference for group',
                                grp['GroupName'],
                                'and user',
                                usr['UserName'],
                                'designated primary when it should not be.'))
                    self._dbgPrint('Successfully validated user/group relationship.')
        
    def test_p_user_references_associators_match(self):
        """
        This test fetches all references and associators for every TestAssoc_User
        object, then makes sure each reference antecedent has an associated
        entry in the list of associators.
        Then, for each group in the list of associators, the reverse of the same
        test is performed.
        """
        self._dbgPrint('Validating user references and associators.')
        for uop in self._conn.EnumerateInstanceNames('TestAssoc_User'):
            urefs = self._conn.References(uop, ResultClass='TestAssoc_MemberOfGroup')
            uassocs = self._conn.Associators(uop, AssocClass='TestAssoc_MemberOfGroup')
            self._dbgPrint('Matching references and associators for user object path %s...' % uop)
            for grp in uassocs:
                grpFound = False
                for i in xrange(len(urefs)):
                    if isObjPathMatch(urefs[i]['Antecedent'], grp.path):
                        urefs.pop(i)
                        grpFound = True
                        break
                if not grpFound:
                    self.fail('Could not find group "%s" in references.' % grp.path)
            if 0 < len(urefs):
                self.fail('Unmatched reference object found.')
            self._dbgPrint('All references accounted for.')
            for grp in uassocs:
                grefs = self._conn.References(grp.path, ResultClass='TestAssoc_MemberOfGroup')
                gassocs = self._conn.AssociatorNames(grp.path, AssocClass='TestAssoc_MemberOfGroup')
                self._dbgPrint('Matching references and associators for group %s...' % grp['GroupName'])
                for uop2 in gassocs:
                    usrFound = False
                    for i in xrange(len(grefs)):
                        if isObjPathMatch(grefs[i]['Dependent'], uop2):
                            grefs.pop(i)
                            usrFound = True
                            break
                    if not usrFound:
                        self.fail('Could not find user "%s" in backreferences.' % uop2)
                if 0 < len(grefs):
                    self.fail('Unmatched backreference object found.')
                self._dbgPrint('All references accounted for.')
                    
    def test_p_group_references_associators_match(self):
        """
        This test is like test_p_user_references_associators_match
        except the test begins with TestAssoc_Group objects.
        """
        self._dbgPrint('Validating group references and associators.')
        for gop in self._conn.EnumerateInstanceNames('TestAssoc_Group'):
            grefs = self._conn.References(gop, ResultClass='TestAssoc_MemberOfGroup')
            gassocs = self._conn.Associators(gop, AssocClass='TestAssoc_MemberOfGroup')
            self._dbgPrint('Matching references and associators for group object path %s...' % gop)
            for usr in gassocs:
                usrFound = False
                for i in xrange(len(grefs)):
                    if isObjPathMatch( grefs[i]['Dependent'], usr.path):
                        grefs.pop(i)
                        usrFound = True
                        break
                if not usrFound:
                    self.fail('Could not find user "%s" in references.' % usr.path)
            if 0 < len(grefs):
                self.fail('Unmatched reference object found.')
            self._dbgPrint('All references accounted for.')
            for usr in gassocs:
                urefs = self._conn.References(usr.path, ResultClass='TestAssoc_MemberOfGroup')
                uassocs = self._conn.AssociatorNames(usr.path, AssocClass='TestAssoc_MemberOfGroup')
                self._dbgPrint('Matching referencs and associators for user %s...' % usr['UserName'])
                for gop2 in uassocs:
                    grpFound = False
                    for i in xrange(len(urefs)):
                        if isObjPathMatch( urefs[i]['Antecedent'], gop2):
                            urefs.pop(i)
                            grpFound = True
                            break
                    if not grpFound:
                        self.fail('Could not find group "%s" in backreferences.' % gop2)
                if 0 < len(urefs):
                    self.fail('Unmatched backreference object found.')
                self._dbgPrint('All references accounted for.')
        
    def test_q_references_propertylist(self):
        """
        This test calls References from user and group objects
        in a number of ways.  In each case a PropertyList is provided.
        Each result from References is checked to make sure that only
        the properties requested are provided.
        """
        self._dbgPrint('Validating references with PropertyList specifications.')
        for uop in self._conn.EnumerateInstanceNames('TestAssoc_User'):
            self._dbgPrint('First validation reference properties for reference to user %s...' % uop)
            for ref in self._conn.References(uop, ResultClass='TestAssoc_MemberOfGroup', PropertyList=['Dependent']):
                try:
                    if ref['Dependent'] is None:
                        self.fail('Reference dependent not set when it should be.')
                    if not isObjPathMatch(ref['Dependent'], uop):
                        self.fail('Reference dependent set to "%s" when it should be "%s"' % (ref['Dependent'], uop))
                except KeyError:
                    self.fail('KeyError caught trying to read "Dependent" property that should exist.')
                isValid=False
                try:
                    # ATTN: This fails since we always include keys.
                    #if ref['Antecedent'] is not None and len(ref['Antecedent']) > 0:
                    #    self.fail('Reference antecedent is set when it should not be (%s).' % ref['Antecedent'])
                    #if ref['isPrimaryGroup'] is not None:
                    #    self.fail('Reference property "isPrimaryGroup" is set when it should not be (%s).' % ref['isPrimaryGroup'])
                    isValid=True
                except KeyError:
                    # not an error
                    isValid=True
                if not isValid:
                    self.fail('Values found for properties not in property list.')
            self._dbgPrint('All reference properties successfully validated.')
            # Figure out the object path of this user's primary group.
            pgop = None
            for ref in self._conn.References(uop, ResultClass='TestAssoc_MemberOfGroup'):
                if ref['isPrimaryGroup']:
                    pgop = ref['Antecedent']
                    break
            # Rescan references with propertylist=Antecedent, isPrimaryGroup.  Check that isPrimaryGroup is correct.
            self._dbgPrint('Second validation reference properties for reference to user %s...' % uop)
            for ref in self._conn.References(uop, ResultClass='TestAssoc_MemberOfGroup',
                    PropertyList=['Antecedent','isPrimaryGroup']):
                try:
                    if ref['Antecedent'] is None:
                        self.fail('Reference antecedent not set when it should be.')
                    if ref['isPrimaryGroup'] is None:
                        self.fail('Reference property "isPrimaryGroup" not set when it should be.')
                    if isObjPathMatch(ref['Antecedent'], pgop) and \
                            not ref['isPrimaryGroup']:
                        self.fail('Reference not designated as primary when it should be.')
                    elif not isObjPathMatch(ref['Antecedent'], pgop) and \
                            ref['isPrimaryGroup']:
                        self.fail('Reference designated as primary when it should not be.')
                except KeyError:
                    self.fail('KeyError caught trying to read properties that should exist.')
                isValid=False
                try:
                    # This fails since we always include the keys.
                    #if ref['Dependent'] is not None and len(ref['Dependent']) > 0:
                    #    self.fail('Reference dependent is set when it should not be (%s).' % ref['Dependent'])
                    isValid=True
                except KeyError:
                    # not an error
                    isValid=True
                if not isValid:
                    self.fail('Values found for properties not in property list.')
            self._dbgPrint('All reference properties successfully validated.')
        for gop in self._conn.EnumerateInstanceNames('TestAssoc_Group'):
            self._dbgPrint('First validation reference properties for reference to group %s...' % gop)
            for ref in self._conn.References(gop, ResultClass='TestAssoc_MemberOfGroup',
                    PropertyList=['Antecedent']):
                try:
                    if ref['Antecedent'] is None:
                        self.fail('Reference antecedent not set when it should be.')
                except KeyERror:
                    self.fail('KeyError caught trying to read antecedent property that should exist.')
                isValid=False
                try:
                    #if ref['Dependent'] is not None and len(ref['Dependent']) > 0:
                    #    self.fail('Reference dependent is set when it should not be (%s).' % ref['Dependent'])
                    isValid=True
                except KeyError:
                    # not an error
                    isValid=True
                if not isValid:
                    self.fail('Values found for properties not in property list.')
            uopmembers = []
            for ref in self._conn.References(gop, ResultClass='TestAssoc_MemberOfGroup'):
                if ref['isPrimaryGroup']:
                    uopmembers.append(ref['Dependent'])
            if 0 < len(uopmembers):
                for ref in self._conn.References(gop, ResultClass='TestAssoc_MemberOfGroup',
                        PropertyList=['Dependent','isPrimaryGroup']):
                    self._dbgPrint('Second validation reference properties for reference to group %s...' % gop)
                    try:
                        if ref['Dependent'] is None:
                            self.fail('Reference dependent not set when it should be.')
                        if ref['isPrimaryGroup'] is None:
                            self.fail('Reference property "isPrimaryGroup" not set when it should be.')
                        if ref['Dependent'] in uopmembers:
                            if not ref['isPrimaryGroup']:
                                self.fail('Reference not designated as primary when it should be.')
                        elif ref['isPrimaryGroup']:
                            self.fail('Reference designated as primary when it should not be.')
                    except KeyError:
                        self.fail('KeyError caught trying to read properties that should exist.')
                    isValid=False
                    try:
                        #if ref['Antecedent'] is not None and len(ref['Antecedent']) > 0:
                        #    self.fail('Reference antecedent is set when it should not be (%s).' % ref['Antecedent'])
                        isValid=True
                    except KeyError:
                        # not an error
                        isValid=True
                    if not isValid:
                        self.fail('Values found for properties not in property list.')
            self._dbgPrint('All reference properties successfully validated.')
        
    def test_r_associators_propertylist(self):
        """
        This test calls Associators from user and group objects in a variety of
        ways, passing a PropertyList into the call each time.  Resulting objects
        are checked to make sure that the property list is honored.
        """
        self._dbgPrint('Validating associators with property list provided.')
        for uop in self._conn.EnumerateInstanceNames('TestAssoc_User'):
            self._dbgPrint('Validating associators properties for user %s...' % uop)
            for grp in self._conn.Associators(uop, AssocClass='TestAssoc_MemberOfGroup',
                    PropertyList=['GroupID']):
                try:
                    if grp['GroupID'] is None:
                        self.fail('GroupID property not set when it should be.')
                except KeyError:
                    self.fail('KeyError caught trying to access property that should exist.')
                isValid=False
                try:
                    if grp['GroupName'] is not None and 0 < len(grp['GroupName']):
                        self.fail('GroupName property set when it should not be.')
                    isValid=True
                except KeyError:
                    # not an error
                    isValid=True
                if not isValid:
                    self.fail('Found properties set that should not be.')
            self._dbgPrint('All associator properties successfully validated.')
        for gop in self._conn.EnumerateInstanceNames('TestAssoc_Group'):
            self._dbgPrint('Validating associators properites for group %s...' % gop)
            for usr in self._conn.Associators(gop, AssocClass='TestAssoc_MemberOfGroup',
                    PropertyList=['UserName','LoginShell']):
                try:
                    if usr['UserName'] is None:
                        self.fail('UserName property not set when it should be.')
                    if usr['LoginShell'] is None:
                        self.fail('LoginShell property not set when it should be.')
                except KeyError:
                    self.fail('KeyError caught trying to access properties that should exist.')
                isValid=False
                try:
                    # some CIMOMs always return keys .
                    #if usr['UserID'] is not None:
                    #    self.fail('UserID property set when it should not be.')
                    if usr['HomeDirectory'] is not None:
                        self.fail('HomeDirectory property set when it should not be.')
                    isValid=True
                except KeyError:
                    # not an error
                    isValid=True
                if not isValid:
                    self.fail('Found properties set that should not be.')
            self._dbgPrint('All associator properties successfully validated.')
        

if __name__ == '__main__':
    parser = optparse.OptionParser()
    wbem_connection.getWBEMConnParserOptions(parser)
    parser.add_option('--level',
            '-l',
            action='store',
            type='int',
            dest='dbglevel',
            help='Indicate the level of debugging statements to display (default=2)',
            default=2)
    parser.add_option('--verbose', '', action='store_true', default=False,
            help='Show verbose output')
    options, arguments = parser.parse_args()
    
    _globalVerbose = options.verbose

    conn = wbem_connection.WBEMConnFromOptions(parser)
    suite = unittest.makeSuite(TestAssociations)
    unittest.TextTestRunner(verbosity=options.dbglevel).run(suite)



import pywbem
from pywbem.cim_provider2 import CIMProvider2

class Test_NullArray(CIMProvider2):
    def __init__ (self, env):
        logger = env.get_logger()
        logger.log_debug('Initializing provider %s from %s' \
                % (self.__class__.__name__, __file__))
        self.instances = {
            '1': {
                'Description' : 'Instance with non-empty properties.',
                'OperationalStatus' : [ self.Values.OperationalStatus.OK],
                'StatusDescriptions' : [ 'OK' ]
            },
            '2': {
                'Description' : 'Instance with empty properties.',
                'OperationalStatus' : pywbem.CIMProperty(
                        name='OperationalStatus',
                        value=[],
                        type='uint16',
                        is_array=True,
                        array_size=0),
                'StatusDescriptions' : pywbem.CIMProperty(
                        name='StatusDescriptions',
                        value=[],
                        type='string',
                        is_array=True,
                        array_size=0)
            },
            '3': {
                'Description' : 'Instance with NULL properties.',
                'OperationalStatus' : pywbem.CIMProperty(
                        name='OperationalStatus',
                        value=None,
                        type='uint16',
                        is_array=True,
                        array_size=0),
                'StatusDescriptions' : pywbem.CIMProperty(
                        name='StatusDescriptions',
                        value=None,
                        type='string',
                        is_array=True,
                        array_size=0)
            }
        }

    def get_instance(self, env, model):
        logger = env.get_logger()
        logger.log_debug('Entering %s.get_instance()' \
                % self.__class__.__name__)

        instance = self.instances[model['InstanceID']]

        model['Description'] = instance['Description']
        model['OperationalStatus'] = instance['OperationalStatus']
        model['StatusDescriptions'] = instance['StatusDescriptions']
        return model

    def enum_instances(self, env, model, keys_only):
        logger = env.get_logger()
        logger.log_debug('Entering %s.enum_instances()' \
                % self.__class__.__name__)
                
        model.path.update({'InstanceID': None})
        
        for k in self.instances.keys():
            model['InstanceID'] = k
            if keys_only:
                yield model
            else:
                try:
                    yield self.get_instance(env, model)
                except pywbem.CIMError, (num, msg):
                    if num not in (pywbem.CIM_ERR_NOT_FOUND, 
                                   pywbem.CIM_ERR_ACCESS_DENIED):
                        raise

    class Values(object):
        class OperationalStatus(object):
            Unknown = pywbem.Uint16(0)
            Other = pywbem.Uint16(1)
            OK = pywbem.Uint16(2)
            Degraded = pywbem.Uint16(3)
            Stressed = pywbem.Uint16(4)
            Predictive_Failure = pywbem.Uint16(5)
            Error = pywbem.Uint16(6)
            Non_Recoverable_Error = pywbem.Uint16(7)
            Starting = pywbem.Uint16(8)
            Stopping = pywbem.Uint16(9)
            Stopped = pywbem.Uint16(10)
            In_Service = pywbem.Uint16(11)
            No_Contact = pywbem.Uint16(12)
            Lost_Communication = pywbem.Uint16(13)
            Aborted = pywbem.Uint16(14)
            Dormant = pywbem.Uint16(15)
            Supporting_Entity_in_Error = pywbem.Uint16(16)
            Completed = pywbem.Uint16(17)
            Power_Mode = pywbem.Uint16(18)
            Relocating = pywbem.Uint16(19)
            # DMTF_Reserved = ..
            # Vendor_Reserved = 0x8000..
            _reverse_map = {0: 'Unknown', 1: 'Other', 2: 'OK', 3: 'Degraded', 4: 'Stressed', 5: 'Predictive Failure', 6: 'Error', 7: 'Non-Recoverable Error', 8: 'Starting', 9: 'Stopping', 10: 'Stopped', 11: 'In Service', 12: 'No Contact', 13: 'Lost Communication', 14: 'Aborted', 15: 'Dormant', 16: 'Supporting Entity in Error', 17: 'Completed', 18: 'Power Mode', 19: 'Relocating'}

def get_providers(env): 
    test_nullarray_prov = Test_NullArray(env)  
    return {'Test_NullArray': test_nullarray_prov}

